@@ -1,6 +1,5 @@
 /* GStreamer
  * Copyright (C) 2005 Wim Taymans <wim@fluendo.com>
- * Copyright (C) 2006 Tim-Philipp Müller <tim centricular net>
  *
  * gstalsasink.c:
  *
@@ -20,27 +19,6 @@
  * Boston, MA 02111-1307, USA.
  */
 
-/**
- * SECTION:element-alsasink
- * @short_description: play audio to an ALSA device
- * @see_also: alsasrc, alsamixer
- *
- * <refsect2>
- * <para>
- * This element renders raw audio samples using the ALSA api.
- * </para>
- * <title>Example pipelines</title>
- * <para>
- * Play an Ogg/Vorbis file.
- * </para>
- * <programlisting>
- * gst-launch -v filesrc location=sine.ogg ! oggdemux ! vorbisdec ! audioconvert ! audioresample ! alsasink
- * </programlisting>
- * </refsect2>
- *
- * Last reviewed on 2006-03-01 (0.10.4)
- */
-
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
@@ -54,13 +32,12 @@
 
 #include "gstalsa.h"
 #include "gstalsasink.h"
-#include "gstalsadeviceprobe.h"
 
 #include <gst/gst-i18n-plugin.h>
 
 /* elementfactory information */
-static const GstElementDetails gst_alsasink_details =
-GST_ELEMENT_DETAILS ("Audio sink (ALSA)",
+static GstElementDetails gst_alsasink_details =
+GST_ELEMENT_DETAILS ("Audio Sink (ALSA)",
     "Sink/Audio",
     "Output to a sound card via ALSA",
     "Wim Taymans <wim@fluendo.com>");
@@ -75,11 +52,9 @@ enum
   PROP_DEVICE_NAME
 };
 
-static void gst_alsasink_init_interfaces (GType type);
-
-GST_BOILERPLATE_FULL (GstAlsaSink, gst_alsasink, GstAudioSink,
-    GST_TYPE_AUDIO_SINK, gst_alsasink_init_interfaces);
-
+static void gst_alsasink_base_init (gpointer g_class);
+static void gst_alsasink_class_init (GstAlsaSinkClass * klass);
+static void gst_alsasink_init (GstAlsaSink * alsasink);
 static void gst_alsasink_dispose (GObject * object);
 static void gst_alsasink_finalise (GObject * object);
 static void gst_alsasink_set_property (GObject * object,
@@ -99,10 +74,11 @@ static guint gst_alsasink_write (GstAudioSink * asink, gpointer data,
 static guint gst_alsasink_delay (GstAudioSink * asink);
 static void gst_alsasink_reset (GstAudioSink * asink);
 
-static gint output_ref;         /* 0    */
-static snd_output_t *output;    /* NULL */
-static GStaticMutex output_mutex = G_STATIC_MUTEX_INIT;
-
+/* AlsaSink signals and args */
+enum
+{
+  LAST_SIGNAL
+};
 
 #if (G_BYTE_ORDER == G_LITTLE_ENDIAN)
 # define ALSA_SINK_FACTORY_ENDIANNESS	"LITTLE_ENDIAN, BIG_ENDIAN"
@@ -115,24 +91,48 @@ static GstStaticPadTemplate alsasink_sink_factory =
     GST_PAD_SINK,
     GST_PAD_ALWAYS,
     GST_STATIC_CAPS ("audio/x-raw-int, "
-        "endianness = (int) { " ALSA_SINK_FACTORY_ENDIANNESS " }, "
-        "signed = (boolean) { TRUE, FALSE }, "
-        "width = (int) 32, "
-        "depth = (int) 32, "
-        "rate = (int) [ 1, MAX ], " "channels = (int) [ 1, MAX ]; "
-        "audio/x-raw-int, "
         "endianness = (int) { " ALSA_SINK_FACTORY_ENDIANNESS " }, "
         "signed = (boolean) { TRUE, FALSE }, "
         "width = (int) 16, "
         "depth = (int) 16, "
-        "rate = (int) [ 1, MAX ], " "channels = (int) [ 1, MAX ]; "
+        "rate = (int) [ 1, MAX ], " "channels = (int) [ 1, 2 ]; "
         "audio/x-raw-int, "
         "signed = (boolean) { TRUE, FALSE }, "
         "width = (int) 8, "
         "depth = (int) 8, "
-        "rate = (int) [ 1, MAX ], " "channels = (int) [ 1, MAX ]")
+        "rate = (int) [ 1, MAX ], " "channels = (int) [ 1, 2 ]")
     );
 
+static GstElementClass *parent_class = NULL;
+
+/* static guint gst_alsasink_signals[LAST_SIGNAL] = { 0 }; */
+
+GType
+gst_alsasink_get_type (void)
+{
+  static GType alsasink_type = 0;
+
+  if (!alsasink_type) {
+    static const GTypeInfo alsasink_info = {
+      sizeof (GstAlsaSinkClass),
+      gst_alsasink_base_init,
+      NULL,
+      (GClassInitFunc) gst_alsasink_class_init,
+      NULL,
+      NULL,
+      sizeof (GstAlsaSink),
+      0,
+      (GInstanceInitFunc) gst_alsasink_init,
+    };
+
+    alsasink_type =
+        g_type_register_static (GST_TYPE_AUDIO_SINK, "GstAlsaSink",
+        &alsasink_info, 0);
+  }
+
+  return alsasink_type;
+}
+
 static void
 gst_alsasink_dispose (GObject * object)
 {
@@ -146,22 +146,6 @@ gst_alsasink_finalise (GObject * object)
 
   g_free (sink->device);
   g_mutex_free (sink->alsa_lock);
-
-  g_static_mutex_lock (&output_mutex);
-  --output_ref;
-  if (output_ref == 0) {
-    snd_output_close (output);
-    output = NULL;
-  }
-  g_static_mutex_unlock (&output_mutex);
-
-  G_OBJECT_CLASS (parent_class)->finalize (object);
-}
-
-static void
-gst_alsasink_init_interfaces (GType type)
-{
-  gst_alsa_type_add_device_property_probe_interface (type);
 }
 
 static void
@@ -189,7 +173,7 @@ gst_alsasink_class_init (GstAlsaSinkClass * klass)
   gstbaseaudiosink_class = (GstBaseAudioSinkClass *) klass;
   gstaudiosink_class = (GstAudioSinkClass *) klass;
 
-  parent_class = g_type_class_peek_parent (klass);
+  parent_class = g_type_class_ref (GST_TYPE_BASE_AUDIO_SINK);
 
   gobject_class->dispose = GST_DEBUG_FUNCPTR (gst_alsasink_dispose);
   gobject_class->finalize = GST_DEBUG_FUNCPTR (gst_alsasink_finalise);
@@ -227,12 +211,9 @@ gst_alsasink_set_property (GObject * object, guint prop_id,
 
   switch (prop_id) {
     case PROP_DEVICE:
-      g_free (sink->device);
-      sink->device = g_value_dup_string (value);
-      /* setting NULL restores the default device */
-      if (sink->device == NULL) {
-        sink->device = g_strdup (DEFAULT_DEVICE);
-      }
+      if (sink->device)
+        g_free (sink->device);
+      sink->device = g_strdup (g_value_get_string (value));
       break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
@@ -270,22 +251,24 @@ gst_alsasink_get_property (GObject * object, guint prop_id,
   }
 }
 
+static snd_output_t *output = NULL;
+
 static void
-gst_alsasink_init (GstAlsaSink * alsasink, GstAlsaSinkClass * g_class)
+gst_alsasink_init (GstAlsaSink * alsasink)
 {
   GST_DEBUG_OBJECT (alsasink, "initializing alsasink");
 
   alsasink->device = g_strdup (DEFAULT_DEVICE);
   alsasink->handle = NULL;
-  alsasink->cached_caps = NULL;
   alsasink->alsa_lock = g_mutex_new ();
 
-  g_static_mutex_lock (&output_mutex);
-  if (output_ref == 0) {
-    snd_output_stdio_attach (&output, stdout, 0);
-    ++output_ref;
-  }
-  g_static_mutex_unlock (&output_mutex);
+  snd_output_stdio_attach (&output, stdout, 0);
+}
+
+static GstCaps *
+gst_alsasink_getcaps (GstBaseSink * bsink)
+{
+  return NULL;
 }
 
 #define CHECK(call, error) \
@@ -294,59 +277,18 @@ if ((err = call) < 0)           \
   goto error;                   \
 } G_STMT_END;
 
-static GstCaps *
-gst_alsasink_getcaps (GstBaseSink * bsink)
-{
-  GstElementClass *element_class;
-  GstPadTemplate *pad_template;
-  GstAlsaSink *sink = GST_ALSA_SINK (bsink);
-  GstCaps *caps;
-
-  if (sink->handle == NULL) {
-    GST_DEBUG_OBJECT (sink, "device not open, using template caps");
-    return NULL;                /* base class will get template caps for us */
-  }
-
-  if (sink->cached_caps) {
-    GST_LOG_OBJECT (sink, "Returning cached caps");
-    return gst_caps_ref (sink->cached_caps);
-  }
-
-  element_class = GST_ELEMENT_GET_CLASS (sink);
-  pad_template = gst_element_class_get_pad_template (element_class, "sink");
-  g_return_val_if_fail (pad_template != NULL, NULL);
-
-  caps = gst_alsa_probe_supported_formats (GST_OBJECT (sink), sink->handle,
-      gst_pad_template_get_caps (pad_template));
-
-  if (caps) {
-    sink->cached_caps = gst_caps_ref (caps);
-  }
-
-  GST_INFO_OBJECT (sink, "returning caps %" GST_PTR_FORMAT, caps);
-
-  return caps;
-}
-
 static int
 set_hwparams (GstAlsaSink * alsa)
 {
   guint rrate;
   gint err, dir;
   snd_pcm_hw_params_t *params;
-  guint period_time, buffer_time;
 
   snd_pcm_hw_params_alloca (&params);
 
-  GST_DEBUG_OBJECT (alsa, "Negotiating to %d channels @ %d Hz (format = %s)",
-      alsa->channels, alsa->rate, snd_pcm_format_name (alsa->format));
+  GST_DEBUG_OBJECT (alsa, "Negotiating to %d channels @ %d Hz", alsa->channels,
+      alsa->rate);
 
-  /* start with requested values, if we cannot configure alsa for those values,
-   * we set these values to -1, which will leave the default alsa values */
-  buffer_time = alsa->buffer_time;
-  period_time = alsa->period_time;
-
-retry:
   /* choose all parameters */
   CHECK (snd_pcm_hw_params_any (alsa->handle, params), no_config);
   /* set the interleaved read/write format */
@@ -365,67 +307,26 @@ retry:
   if (rrate != alsa->rate)
     goto rate_match;
 
-  /* get and dump some limits */
-  {
-    guint min, max;
-
-    snd_pcm_hw_params_get_buffer_time_min (params, &min, &dir);
-    snd_pcm_hw_params_get_buffer_time_max (params, &max, &dir);
-
-    GST_DEBUG_OBJECT (alsa, "buffer time %u, min %u, max %u",
-        alsa->buffer_time, min, max);
-
-    snd_pcm_hw_params_get_period_time_min (params, &min, &dir);
-    snd_pcm_hw_params_get_period_time_max (params, &max, &dir);
-
-    GST_DEBUG_OBJECT (alsa, "period time %u, min %u, max %u",
-        alsa->period_time, min, max);
-
-    snd_pcm_hw_params_get_periods_min (params, &min, &dir);
-    snd_pcm_hw_params_get_periods_max (params, &max, &dir);
-
-    GST_DEBUG_OBJECT (alsa, "periods min %u, max %u", min, max);
-  }
-
-  /* now try to configure the buffer time and period time, if one
-   * of those fail, we fall back to the defaults and emit a warning. */
-  if (buffer_time != -1) {
+  if (alsa->buffer_time != -1) {
     /* set the buffer time */
-    if ((err = snd_pcm_hw_params_set_buffer_time_near (alsa->handle, params,
-                &buffer_time, &dir)) < 0) {
-      GST_ELEMENT_WARNING (alsa, RESOURCE, SETTINGS, (NULL),
-          ("Unable to set buffer time %i for playback: %s",
-              buffer_time, snd_strerror (err)));
-      /* disable buffer_time the next round */
-      buffer_time = -1;
-      goto retry;
-    }
+    CHECK (snd_pcm_hw_params_set_buffer_time_near (alsa->handle, params,
+            &alsa->buffer_time, &dir), buffer_time);
   }
-  if (period_time != -1) {
+  if (alsa->period_time != -1) {
     /* set the period time */
-    if ((err = snd_pcm_hw_params_set_period_time_near (alsa->handle, params,
-                &period_time, &dir)) < 0) {
-      GST_ELEMENT_WARNING (alsa, RESOURCE, SETTINGS, (NULL),
-          ("Unable to set period time %i for playback: %s",
-              period_time, snd_strerror (err)));
-      /* disable period_time the next round */
-      period_time = -1;
-      goto retry;
-    }
+    CHECK (snd_pcm_hw_params_set_period_time_near (alsa->handle, params,
+            &alsa->period_time, &dir), period_time);
   }
 
   /* write the parameters to device */
   CHECK (snd_pcm_hw_params (alsa->handle, params), set_hw_params);
 
-  /* now get the configured values */
   CHECK (snd_pcm_hw_params_get_buffer_size (params, &alsa->buffer_size),
       buffer_size);
+
   CHECK (snd_pcm_hw_params_get_period_size (params, &alsa->period_size, &dir),
       period_size);
 
-  GST_DEBUG_OBJECT (alsa, "buffer size %u, period size %u", alsa->buffer_size,
-      alsa->period_size);
-
   return 0;
 
   /* ERRORS */
@@ -478,12 +379,26 @@ rate_match:
         ("Rate doesn't match (requested %iHz, get %iHz)", alsa->rate, err));
     return -EINVAL;
   }
+buffer_time:
+  {
+    GST_ELEMENT_ERROR (alsa, RESOURCE, SETTINGS, (NULL),
+        ("Unable to set buffer time %i for playback: %s",
+            alsa->buffer_time, snd_strerror (err)));
+    return err;
+  }
 buffer_size:
   {
     GST_ELEMENT_ERROR (alsa, RESOURCE, SETTINGS, (NULL),
         ("Unable to get buffer size for playback: %s", snd_strerror (err)));
     return err;
   }
+period_time:
+  {
+    GST_ELEMENT_ERROR (alsa, RESOURCE, SETTINGS, (NULL),
+        ("Unable to set period time %i for playback: %s", alsa->period_time,
+            snd_strerror (err)));
+    return err;
+  }
 period_size:
   {
     GST_ELEMENT_ERROR (alsa, RESOURCE, SETTINGS, (NULL),
@@ -565,10 +480,6 @@ alsasink_parse_spec (GstAlsaSink * alsa, GstRingBufferSpec * spec)
 {
   switch (spec->type) {
     case GST_BUFTYPE_LINEAR:
-      GST_DEBUG_OBJECT (alsa,
-          "Linear format : depth=%d, width=%d, sign=%d, bigend=%d", spec->depth,
-          spec->width, spec->sign, spec->bigend);
-
       alsa->format = snd_pcm_build_linear_format (spec->depth, spec->width,
           spec->sign ? 0 : 1, spec->bigend ? 1 : 0);
       break;
@@ -632,12 +543,10 @@ gst_alsasink_open (GstAudioSink * asink)
 open_error:
   {
     if (err == -EBUSY) {
-      GST_ELEMENT_ERROR (alsa, RESOURCE, BUSY, (NULL), ("Device '%s' is busy",
-              alsa->device));
+      GST_ELEMENT_ERROR (alsa, RESOURCE, BUSY, (NULL), (NULL));
     } else {
       GST_ELEMENT_ERROR (alsa, RESOURCE, OPEN_WRITE,
-          (NULL), ("Playback open error on device '%s': %s", alsa->device,
-              snd_strerror (err)));
+          (NULL), ("Playback open error: %s", snd_strerror (err)));
     }
     return FALSE;
   }
@@ -662,6 +571,10 @@ gst_alsasink_prepare (GstAudioSink * asink, GstRingBufferSpec * spec)
   alsa->bytes_per_sample = spec->bytes_per_sample;
   spec->segsize = alsa->period_size * spec->bytes_per_sample;
   spec->segtotal = alsa->buffer_size / alsa->period_size;
+  spec->silence_sample[0] = 0;
+  spec->silence_sample[1] = 0;
+  spec->silence_sample[2] = 0;
+  spec->silence_sample[3] = 0;
 
   return TRUE;
 
@@ -737,7 +650,6 @@ gst_alsasink_close (GstAudioSink * asink)
 
   CHECK (snd_pcm_close (alsa->handle), close_error);
   alsa->handle = NULL;
-  gst_caps_replace (&alsa->cached_caps, NULL);
 
   return TRUE;
 
@@ -858,14 +770,12 @@ drop_error:
   {
     GST_ELEMENT_ERROR (alsa, RESOURCE, SETTINGS,
         ("alsa-reset: pcm drop error: %s", snd_strerror (err)), (NULL));
-    GST_ALSA_UNLOCK (asink);
     return;
   }
 prepare_error:
   {
     GST_ELEMENT_ERROR (alsa, RESOURCE, SETTINGS,
         ("alsa-reset: pcm prepare error: %s", snd_strerror (err)), (NULL));
-    GST_ALSA_UNLOCK (asink);
     return;
   }
 }
