@@ -3,7 +3,7 @@
  *                    2000 Wim Taymans <wtay@chello.be>
  *                    2001 Bastien Nocera <hadess@hadess.net>
  *                    2003 Colin Walters <walters@verbum.org>
- *                    2005 Tim-Philipp M??ller <tim centricular net>
+ *                    2005 Tim-Philipp Müller <tim centricular net>
  *
  * gstgnomevfssink.c: 
  *
@@ -50,14 +50,8 @@
  * The above pipeline will re-encode an mp3 file into FLAC format and store
  * it on a remote host using the Samba protocol.
  * </para>
- * <para>
- * Applications can connect to the allow-overwrite signal to receive a callback when an
- * existing file will be overwritten. The return value of the signal will determine if
- * gnomevfssink will overwrite the resource or abort with an error.
- * </para>
  * </refsect2>
  *
- * Last reviewed on 2006-02-28 (0.10.4)
  */
 
 #ifdef HAVE_CONFIG_H
@@ -73,12 +67,6 @@
 #include <string.h>
 #include <errno.h>
 
-static GstElementDetails gst_gnome_vfs_sink_details =
-GST_ELEMENT_DETAILS ("GnomeVFS Sink",
-    "Sink/File",
-    "Write a stream to a GnomeVFS URI",
-    "Bastien Nocera <hadess@hadess.net>");
-
 enum
 {
   SIGNAL_ERASE_ASK,
@@ -145,6 +133,11 @@ static void
 gst_gnome_vfs_sink_base_init (gpointer g_class)
 {
   GstElementClass *element_class = GST_ELEMENT_CLASS (g_class);
+  static GstElementDetails gst_gnome_vfs_sink_details =
+      GST_ELEMENT_DETAILS ("GnomeVFS Sink",
+      "Sink/File",
+      "Write a stream to a GnomeVFS URI",
+      "Bastien Nocera <hadess@hadess.net>");
 
   gst_element_class_add_pad_template (element_class,
       gst_static_pad_template_get (&sinktemplate));
@@ -179,9 +172,8 @@ gst_gnome_vfs_sink_class_init (GstGnomeVFSSinkClass * klass)
   gobject_class->get_property = gst_gnome_vfs_sink_get_property;
   gobject_class->finalize = gst_gnome_vfs_sink_finalize;
 
-  g_object_class_install_property (gobject_class, ARG_LOCATION,
-      g_param_spec_string ("location", "File Location",
-          "Location of the file to write", NULL, G_PARAM_READWRITE));
+  gst_element_class_install_std_props (GST_ELEMENT_CLASS (klass),
+      "location", ARG_LOCATION, G_PARAM_READWRITE, NULL);
   g_object_class_install_property (gobject_class, ARG_URI,
       g_param_spec_boxed ("uri", "GnomeVFSURI", "URI for GnomeVFS",
           GST_TYPE_GNOME_VFS_URI, G_PARAM_READWRITE));
@@ -190,17 +182,6 @@ gst_gnome_vfs_sink_class_init (GstGnomeVFSSinkClass * klass)
           "GnomeVFSHandle", "Handle for GnomeVFS",
           GST_TYPE_GNOME_VFS_HANDLE, G_PARAM_READWRITE));
 
-  /**
-   * GstGnomeVFSSink::allow-overwrite
-   * @sink: the object which received the signal
-   * @uri: the URI to be overwritten
-   *
-   * This signal is fired when gnomevfssink is about to overwrite an
-   * existing resource. The application can connect to this signal and ask
-   * the user if the resource may be overwritten. 
-   *
-   * Returns: A boolean indicating that the resource may be overwritten.
-   */
   gst_gnome_vfs_sink_signals[SIGNAL_ERASE_ASK] =
       g_signal_new ("allow-overwrite", G_TYPE_FROM_CLASS (klass),
       G_SIGNAL_RUN_CLEANUP, G_STRUCT_OFFSET (GstGnomeVFSSinkClass, erase_ask),
@@ -267,8 +248,6 @@ gst_gnome_vfs_sink_set_property (GObject * object, guint prop_id,
 
   switch (prop_id) {
     case ARG_LOCATION:{
-      const gchar *new_location;
-
       if (sink->uri) {
         gnome_vfs_uri_unref (sink->uri);
         sink->uri = NULL;
@@ -277,10 +256,8 @@ gst_gnome_vfs_sink_set_property (GObject * object, guint prop_id,
         g_free (sink->uri_name);
         sink->uri_name = NULL;
       }
-
-      new_location = g_value_get_string (value);
-      if (new_location) {
-        sink->uri_name = gst_gnome_vfs_location_to_uri_string (new_location);
+      if (g_value_get_string (value)) {
+        sink->uri_name = g_value_dup_string (value);
         sink->uri = gnome_vfs_uri_new (sink->uri_name);
       }
       break;
@@ -354,13 +331,11 @@ gst_gnome_vfs_sink_open_file (GstGnomeVFSSink * sink)
   GnomeVFSResult result;
 
   if (sink->uri) {
-    /* open the file, all permissions, umask will apply */
+    /* open the file */
     result = gnome_vfs_create_uri (&(sink->handle), sink->uri,
         GNOME_VFS_OPEN_WRITE, TRUE,
-        GNOME_VFS_PERM_USER_READ | GNOME_VFS_PERM_USER_WRITE |
-        GNOME_VFS_PERM_GROUP_READ | GNOME_VFS_PERM_GROUP_WRITE |
-        GNOME_VFS_PERM_OTHER_READ | GNOME_VFS_PERM_OTHER_WRITE);
-
+        GNOME_VFS_PERM_USER_READ | GNOME_VFS_PERM_USER_WRITE
+        | GNOME_VFS_PERM_GROUP_READ);
     /* if the file existed and the property says to ask, then ask! */
     if (result == GNOME_VFS_ERROR_FILE_EXISTS) {
       gboolean erase_anyway = FALSE;
@@ -371,9 +346,8 @@ gst_gnome_vfs_sink_open_file (GstGnomeVFSSink * sink)
       if (erase_anyway) {
         result = gnome_vfs_create_uri (&(sink->handle), sink->uri,
             GNOME_VFS_OPEN_WRITE, FALSE,
-            GNOME_VFS_PERM_USER_READ | GNOME_VFS_PERM_USER_WRITE |
-            GNOME_VFS_PERM_GROUP_READ | GNOME_VFS_PERM_GROUP_WRITE |
-            GNOME_VFS_PERM_OTHER_READ | GNOME_VFS_PERM_OTHER_WRITE);
+            GNOME_VFS_PERM_USER_READ | GNOME_VFS_PERM_USER_WRITE
+            | GNOME_VFS_PERM_GROUP_READ);
       }
     }
 
@@ -558,11 +532,8 @@ gst_gnome_vfs_sink_render (GstBaseSink * basesink, GstBuffer * buf)
     case GNOME_VFS_ERROR_NO_SPACE:{
       /* TODO: emit signal/send msg on out-of-diskspace and
        * handle this gracefully (see open bug) (tpm) */
-      GST_ELEMENT_ERROR (sink, RESOURCE, NO_SPACE_LEFT, (NULL),
-          ("bufsize=%u, written=%u", GST_BUFFER_SIZE (buf), (guint) written));
-      ret = GST_FLOW_ERROR;
-      break;
     }
+      /* fall-through */
     default:{
       gchar *filename = gnome_vfs_uri_to_string (sink->uri,
           GNOME_VFS_URI_HIDE_PASSWORD);
