@@ -25,38 +25,42 @@
 
 /**
  * SECTION:element-gnomevfssrc
+ * @short_description: Read from any GnomeVFS-supported location
  * @see_also: #GstFileSrc, #GstGnomeVFSSink
  *
+ * <refsect2>
+ * <para>
  * This plugin reads data from a local or remote location specified
  * by an URI. This location can be specified using any protocol supported by
  * the GnomeVFS library. Common protocols are 'file', 'http', 'ftp', or 'smb'.
- *
- * In case the #GstGnomeVFSSrc:iradio-mode property is set and the
- * location is a http resource, gnomevfssrc will send special icecast http
- * headers to the server to request additional icecast metainformation. If
- * the server is not an icecast server, it will display the same behaviour
- * as if the #GstGnomeVFSSrc:iradio-mode property was not set. However,
- * if the server is in fact an icecast server, gnomevfssrc will output
- * data with a media type of application/x-icy, in which case you will
- * need to use the #GstICYDemux element as follow-up element to extract
- * the icecast meta data and to determine the underlying media type.
- *
- * <refsect2>
- * <title>Example launch lines</title>
- * |[
+ * </para>
+ * <para>
+ * Example pipeline:
+ * <programlisting>
  * gst-launch -v gnomevfssrc location=file:///home/joe/foo.xyz ! fakesink
- * ]| The above pipeline will simply read a local file and do nothing with the
+ * </programlisting>
+ * The above pipeline will simply read a local file and do nothing with the
  * data read. Instead of gnomevfssrc, we could just as well have used the
  * filesrc element here.
- * |[
+ * </para>
+ * <para>
+ * Another example pipeline:
+ * <programlisting>
  * gst-launch -v gnomevfssrc location=smb://othercomputer/foo.xyz ! filesink location=/home/joe/foo.xyz
- * ]| The above pipeline will copy a file from a remote host to the local file
+ * </programlisting>
+ * The above pipeline will copy a file from a remote host to the local file
  * system using the Samba protocol.
- * |[
+ * </para>
+ * <para>
+ * Yet another example pipeline:
+ * <programlisting>
  * gst-launch -v gnomevfssrc location=http://music.foobar.com/demo.mp3 ! mad ! audioconvert ! audioresample ! alsasink
- * ]| The above pipeline will read and decode and play an mp3 file from a
+ * </programlisting>
+ * The above pipeline will read and decode and play an mp3 file from a
  * web server using the http protocol.
+ * </para>
  * </refsect2>
+ *
  */
 
 
@@ -71,7 +75,6 @@
 
 #include "gstgnomevfssrc.h"
 
-#include <stdio.h>
 #include <stdlib.h>
 #include <sys/types.h>
 #include <sys/socket.h>
@@ -87,25 +90,18 @@
 #include <string.h>
 
 #include <gst/gst.h>
-#include <gst/tag/tag.h>
-
 /* gnome-vfs.h doesn't include the following header, which we need: */
 #include <libgnomevfs/gnome-vfs-standard-callbacks.h>
 
 GST_DEBUG_CATEGORY_STATIC (gnomevfssrc_debug);
 #define GST_CAT_DEFAULT gnomevfssrc_debug
 
-static const GstElementDetails gst_gnome_vfs_src_details =
-GST_ELEMENT_DETAILS ("GnomeVFS Source",
-    "Source/File",
-    "Read from any GnomeVFS-supported file",
-    "Bastien Nocera <hadess@hadess.net>\n"
-    "GStreamer maintainers <gstreamer-devel@lists.sourceforge.net>");
 
 static GStaticMutex count_lock = G_STATIC_MUTEX_INIT;
 static gint ref_count = 0;
 static gboolean vfs_owner = FALSE;
 
+
 static GstStaticPadTemplate srctemplate = GST_STATIC_PAD_TEMPLATE ("src",
     GST_PAD_SRC,
     GST_PAD_ALWAYS,
@@ -138,11 +134,15 @@ static void gst_gnome_vfs_src_get_property (GObject * object, guint prop_id,
 static gboolean gst_gnome_vfs_src_stop (GstBaseSrc * src);
 static gboolean gst_gnome_vfs_src_start (GstBaseSrc * src);
 static gboolean gst_gnome_vfs_src_is_seekable (GstBaseSrc * src);
-static gboolean gst_gnome_vfs_src_check_get_range (GstBaseSrc * src);
 static gboolean gst_gnome_vfs_src_get_size (GstBaseSrc * src, guint64 * size);
 static GstFlowReturn gst_gnome_vfs_src_create (GstBaseSrc * basesrc,
     guint64 offset, guint size, GstBuffer ** buffer);
-static gboolean gst_gnome_vfs_src_query (GstBaseSrc * src, GstQuery * query);
+
+static int audiocast_init (GstGnomeVFSSrc * src);
+static int audiocast_register_listener (gint * port, gint * fd);
+static void audiocast_do_notifications (GstGnomeVFSSrc * src);
+static gpointer audiocast_thread_run (GstGnomeVFSSrc * src);
+static void audiocast_thread_kill (GstGnomeVFSSrc * src);
 
 static GstElementClass *parent_class = NULL;
 
@@ -182,6 +182,12 @@ static void
 gst_gnome_vfs_src_base_init (gpointer g_class)
 {
   GstElementClass *element_class = GST_ELEMENT_CLASS (g_class);
+  static GstElementDetails gst_gnome_vfs_src_details =
+      GST_ELEMENT_DETAILS ("GnomeVFS Source",
+      "Source/File",
+      "Read from any GnomeVFS-supported file",
+      "Bastien Nocera <hadess@hadess.net>\n"
+      "Ronald S. Bultje <rbultje@ronald.bitfreak.net>");
 
   gst_element_class_add_pad_template (element_class,
       gst_static_pad_template_get (&srctemplate));
@@ -195,9 +201,11 @@ static void
 gst_gnome_vfs_src_class_init (GstGnomeVFSSrcClass * klass)
 {
   GObjectClass *gobject_class;
+  GstElementClass *gstelement_class;
   GstBaseSrcClass *gstbasesrc_class;
 
   gobject_class = G_OBJECT_CLASS (klass);
+  gstelement_class = GST_ELEMENT_CLASS (klass);
   gstbasesrc_class = GST_BASE_SRC_CLASS (klass);
 
   parent_class = g_type_class_peek_parent (klass);
@@ -213,43 +221,40 @@ gst_gnome_vfs_src_class_init (GstGnomeVFSSrcClass * klass)
       ARG_HANDLE,
       g_param_spec_boxed ("handle",
           "GnomeVFSHandle", "Handle for GnomeVFS",
-          GST_TYPE_GNOME_VFS_HANDLE,
-          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+          GST_TYPE_GNOME_VFS_HANDLE, G_PARAM_READWRITE));
 
   /* icecast stuff */
   g_object_class_install_property (gobject_class,
       ARG_IRADIO_MODE,
       g_param_spec_boolean ("iradio-mode",
           "iradio-mode",
-          "Enable internet radio mode (extraction of shoutcast/icecast metadata)",
-          FALSE, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+          "Enable internet radio mode (extraction of icecast/audiocast metadata)",
+          FALSE, G_PARAM_READWRITE));
   g_object_class_install_property (gobject_class,
       ARG_IRADIO_NAME,
       g_param_spec_string ("iradio-name",
-          "iradio-name", "Name of the stream", NULL,
-          G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
-  g_object_class_install_property (gobject_class, ARG_IRADIO_GENRE,
-      g_param_spec_string ("iradio-genre", "iradio-genre",
-          "Genre of the stream", NULL,
-          G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
-  g_object_class_install_property (gobject_class, ARG_IRADIO_URL,
-      g_param_spec_string ("iradio-url", "iradio-url",
-          "Homepage URL for radio stream", NULL,
-          G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
-  g_object_class_install_property (gobject_class, ARG_IRADIO_TITLE,
-      g_param_spec_string ("iradio-title", "iradio-title",
-          "Name of currently playing song", NULL,
-          G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
+          "iradio-name", "Name of the stream", NULL, G_PARAM_READABLE));
+  g_object_class_install_property (gobject_class,
+      ARG_IRADIO_GENRE,
+      g_param_spec_string ("iradio-genre",
+          "iradio-genre", "Genre of the stream", NULL, G_PARAM_READABLE));
+  g_object_class_install_property (gobject_class,
+      ARG_IRADIO_URL,
+      g_param_spec_string ("iradio-url",
+          "iradio-url",
+          "Homepage URL for radio stream", NULL, G_PARAM_READABLE));
+  g_object_class_install_property (gobject_class,
+      ARG_IRADIO_TITLE,
+      g_param_spec_string ("iradio-title",
+          "iradio-title",
+          "Name of currently playing song", NULL, G_PARAM_READABLE));
 
   gstbasesrc_class->start = GST_DEBUG_FUNCPTR (gst_gnome_vfs_src_start);
   gstbasesrc_class->stop = GST_DEBUG_FUNCPTR (gst_gnome_vfs_src_stop);
   gstbasesrc_class->get_size = GST_DEBUG_FUNCPTR (gst_gnome_vfs_src_get_size);
   gstbasesrc_class->is_seekable =
       GST_DEBUG_FUNCPTR (gst_gnome_vfs_src_is_seekable);
-  gstbasesrc_class->check_get_range =
-      GST_DEBUG_FUNCPTR (gst_gnome_vfs_src_check_get_range);
   gstbasesrc_class->create = GST_DEBUG_FUNCPTR (gst_gnome_vfs_src_create);
-  gstbasesrc_class->query = GST_DEBUG_FUNCPTR (gst_gnome_vfs_src_query);
 }
 
 static void
@@ -261,13 +266,20 @@ gst_gnome_vfs_src_init (GstGnomeVFSSrc * gnomevfssrc)
   gnomevfssrc->curoffset = 0;
   gnomevfssrc->seekable = FALSE;
 
+  gnomevfssrc->icy_metaint = 0;
   gnomevfssrc->iradio_mode = FALSE;
   gnomevfssrc->http_callbacks_pushed = FALSE;
+  gnomevfssrc->icy_count = 0;
   gnomevfssrc->iradio_name = NULL;
   gnomevfssrc->iradio_genre = NULL;
   gnomevfssrc->iradio_url = NULL;
   gnomevfssrc->iradio_title = NULL;
 
+  gnomevfssrc->audiocast_udpdata_mutex = g_mutex_new ();
+  gnomevfssrc->audiocast_queue_mutex = g_mutex_new ();
+  gnomevfssrc->audiocast_notify_queue = NULL;
+  gnomevfssrc->audiocast_thread = NULL;
+
   g_static_mutex_lock (&count_lock);
   if (ref_count == 0) {
     /* gnome vfs engine init */
@@ -299,20 +311,13 @@ gst_gnome_vfs_src_finalize (GObject * object)
     src->uri = NULL;
   }
 
-  g_free (src->uri_name);
-  src->uri_name = NULL;
+  if (src->uri_name) {
+    g_free (src->uri_name);
+    src->uri_name = NULL;
+  }
 
-  g_free (src->iradio_name);
-  src->iradio_name = NULL;
-
-  g_free (src->iradio_genre);
-  src->iradio_genre = NULL;
-
-  g_free (src->iradio_url);
-  src->iradio_url = NULL;
-
-  g_free (src->iradio_title);
-  src->iradio_title = NULL;
+  g_mutex_free (src->audiocast_udpdata_mutex);
+  g_mutex_free (src->audiocast_queue_mutex);
 
   G_OBJECT_CLASS (parent_class)->finalize (object);
 }
@@ -321,7 +326,7 @@ gst_gnome_vfs_src_finalize (GObject * object)
  * URI interface support.
  */
 
-static GstURIType
+static guint
 gst_gnome_vfs_src_uri_get_type (void)
 {
   return GST_URI_SRC;
@@ -330,7 +335,12 @@ gst_gnome_vfs_src_uri_get_type (void)
 static gchar **
 gst_gnome_vfs_src_uri_get_protocols (void)
 {
-  return gst_gnomevfs_get_supported_uris ();
+  static gchar **protocols = NULL;
+
+  if (!protocols)
+    protocols = gst_gnomevfs_get_supported_uris ();
+
+  return protocols;
 }
 
 static const gchar *
@@ -371,13 +381,12 @@ gst_gnome_vfs_src_set_property (GObject * object, guint prop_id,
     const GValue * value, GParamSpec * pspec)
 {
   GstGnomeVFSSrc *src;
+  gchar cwd[PATH_MAX];
 
   src = GST_GNOME_VFS_SRC (object);
 
   switch (prop_id) {
-    case ARG_LOCATION:{
-      const gchar *new_location;
-
+    case ARG_LOCATION:
       /* the element must be stopped or paused in order to do this */
       if (GST_STATE (src) == GST_STATE_PLAYING ||
           GST_STATE (src) == GST_STATE_PAUSED)
@@ -392,13 +401,25 @@ gst_gnome_vfs_src_set_property (GObject * object, guint prop_id,
         src->uri_name = NULL;
       }
 
-      new_location = g_value_get_string (value);
-      if (new_location) {
-        src->uri_name = gst_gnome_vfs_location_to_uri_string (new_location);
+      if (g_value_get_string (value)) {
+        const gchar *location = g_value_get_string (value);
+
+        if (!strchr (location, ':')) {
+          gchar *newloc = gnome_vfs_escape_path_string (location);
+
+          if (*newloc == '/')
+            src->uri_name = g_strdup_printf ("file://%s", newloc);
+          else
+            src->uri_name =
+                g_strdup_printf ("file://%s/%s", getcwd (cwd, PATH_MAX),
+                newloc);
+          g_free (newloc);
+        } else
+          src->uri_name = g_strdup (location);
+
         src->uri = gnome_vfs_uri_new (src->uri_name);
       }
       break;
-    }
     case ARG_HANDLE:
       if (GST_STATE (src) == GST_STATE_NULL ||
           GST_STATE (src) == GST_STATE_READY) {
@@ -450,7 +471,9 @@ gst_gnome_vfs_src_get_property (GObject * object, guint prop_id, GValue * value,
       g_value_set_string (value, src->iradio_url);
       break;
     case ARG_IRADIO_TITLE:
+      g_mutex_lock (src->audiocast_udpdata_mutex);
       g_value_set_string (value, src->iradio_title);
+      g_mutex_unlock (src->audiocast_udpdata_mutex);
       break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
@@ -458,14 +481,260 @@ gst_gnome_vfs_src_get_property (GObject * object, guint prop_id, GValue * value,
   }
 }
 
+static char *
+unicodify (const char *str, int len, ...)
+{
+  char *ret = NULL, *cset;
+  va_list args;
+  gsize bytes_read, bytes_written;
+
+  if (g_utf8_validate (str, len, NULL))
+    return g_strndup (str, len >= 0 ? len : strlen (str));
+
+  va_start (args, len);
+  while ((cset = va_arg (args, char *)) != NULL)
+  {
+    if (!strcmp (cset, "locale"))
+      ret = g_locale_to_utf8 (str, len, &bytes_read, &bytes_written, NULL);
+    else
+      ret = g_convert (str, len, "UTF-8", cset,
+          &bytes_read, &bytes_written, NULL);
+    if (ret)
+      break;
+  }
+  va_end (args);
+
+  return ret;
+}
+
 static char *
 gst_gnome_vfs_src_unicodify (const char *str)
 {
-  const gchar *env_vars[] = { "GST_ICY_TAG_ENCODING",
-    "GST_TAG_ENCODING", NULL
-  };
+  return unicodify (str, -1, "locale", "ISO-8859-1", NULL);
+}
 
-  return gst_tag_freeform_string_to_utf8 (str, -1, env_vars);
+/*
+ * icecast/audiocast metadata extraction support code
+ */
+
+static int
+audiocast_init (GstGnomeVFSSrc * src)
+{
+  int pipefds[2];
+  GError *error = NULL;
+
+  if (!src->iradio_mode)
+    return TRUE;
+  GST_DEBUG_OBJECT (src, "audiocast: registering listener");
+  if (audiocast_register_listener (&src->audiocast_port,
+          &src->audiocast_fd) < 0) {
+    GST_ELEMENT_ERROR (src, RESOURCE, OPEN_READ, (NULL),
+        ("Unable to listen on UDP port %d", src->audiocast_port));
+    close (src->audiocast_fd);
+    return FALSE;
+  }
+  GST_DEBUG_OBJECT (src, "audiocast: creating pipe");
+  src->audiocast_notify_queue = NULL;
+  if (pipe (pipefds) < 0) {
+    close (src->audiocast_fd);
+    return FALSE;
+  }
+  src->audiocast_thread_die_infd = pipefds[0];
+  src->audiocast_thread_die_outfd = pipefds[1];
+  GST_DEBUG_OBJECT (src, "audiocast: creating audiocast thread");
+  src->audiocast_thread =
+      g_thread_create ((GThreadFunc) audiocast_thread_run, src, TRUE, &error);
+  if (error != NULL) {
+    GST_ELEMENT_ERROR (src, RESOURCE, TOO_LAZY, (NULL),
+        ("Unable to create thread: %s", error->message));
+    close (src->audiocast_fd);
+    return FALSE;
+  }
+  return TRUE;
+}
+
+static int
+audiocast_register_listener (gint * port, gint * fd)
+{
+  struct sockaddr_in sin;
+  int sock;
+  socklen_t sinlen = sizeof (struct sockaddr_in);
+
+  GST_DEBUG ("audiocast: establishing UDP listener");
+
+  if ((sock = socket (AF_INET, SOCK_DGRAM, 0)) < 0)
+    goto lose;
+
+  memset (&sin, 0, sinlen);
+  sin.sin_family = AF_INET;
+  sin.sin_addr.s_addr = g_htonl (INADDR_ANY);
+
+  if (bind (sock, (struct sockaddr *) &sin, sinlen) < 0)
+    goto lose_and_close;
+
+  memset (&sin, 0, sinlen);
+  if (getsockname (sock, (struct sockaddr *) &sin, &sinlen) < 0)
+    goto lose_and_close;
+
+  GST_DEBUG ("audiocast: listening on local %s:%d", inet_ntoa (sin.sin_addr),
+      g_ntohs (sin.sin_port));
+
+  *port = g_ntohs (sin.sin_port);
+  *fd = sock;
+
+  return 0;
+lose_and_close:
+  close (sock);
+lose:
+  return -1;
+}
+
+static void
+audiocast_do_notifications (GstGnomeVFSSrc * src)
+{
+  /* Send any pending notifications we got from the UDP thread. */
+  if (src->iradio_mode) {
+    GList *entry;
+
+    g_mutex_lock (src->audiocast_queue_mutex);
+    for (entry = src->audiocast_notify_queue; entry; entry = entry->next)
+      g_object_notify (G_OBJECT (src), (const gchar *) entry->data);
+    g_list_free (src->audiocast_notify_queue);
+    src->audiocast_notify_queue = NULL;
+    g_mutex_unlock (src->audiocast_queue_mutex);
+  }
+}
+
+static gpointer
+audiocast_thread_run (GstGnomeVFSSrc * src)
+{
+  char buf[1025], **lines;
+  gsize len;
+  fd_set fdset, readset;
+  struct sockaddr_in from;
+  socklen_t fromlen = sizeof (struct sockaddr_in);
+
+  FD_ZERO (&fdset);
+
+  FD_SET (src->audiocast_fd, &fdset);
+  FD_SET (src->audiocast_thread_die_infd, &fdset);
+
+  while (1) {
+    GST_DEBUG ("audiocast thread: dropping into select");
+    readset = fdset;
+    if (select (FD_SETSIZE, &readset, NULL, NULL, NULL) < 0) {
+      perror ("select");
+      return NULL;
+    }
+    if (FD_ISSET (src->audiocast_thread_die_infd, &readset)) {
+      char buf[1];
+
+      GST_DEBUG ("audiocast thread: got die character");
+      if (read (src->audiocast_thread_die_infd, buf, 1) != 1)
+        g_warning ("gnomevfssrc: could not read from audiocast fd");
+      close (src->audiocast_thread_die_infd);
+      close (src->audiocast_fd);
+      return NULL;
+    }
+    GST_DEBUG ("audiocast thread: reading data");
+    len =
+        recvfrom (src->audiocast_fd, buf, sizeof (buf) - 1, 0,
+        (struct sockaddr *) &from, &fromlen);
+    if (len < 0 && errno == EAGAIN)
+      continue;
+    else if (len >= 0) {
+      int i;
+      char *valptr, *value;
+
+      buf[len] = '\0';
+      lines = g_strsplit (buf, "\n", 0);
+      if (!lines)
+        continue;
+
+      for (i = 0; lines[i]; i++) {
+        while ((lines[i][strlen (lines[i]) - 1] == '\n') ||
+            (lines[i][strlen (lines[i]) - 1] == '\r'))
+          lines[i][strlen (lines[i]) - 1] = '\0';
+
+        valptr = strchr (lines[i], ':');
+
+        if (!valptr)
+          continue;
+        else
+          valptr++;
+
+        g_strstrip (valptr);
+        if (!strlen (valptr))
+          continue;
+
+        value = gst_gnome_vfs_src_unicodify (valptr);
+        if (!value) {
+          g_print ("Unable to convert \"%s\" to UTF-8!\n", valptr);
+          continue;
+        }
+
+        if (!strncmp (lines[i], "x-audiocast-streamtitle", 23)) {
+          g_mutex_lock (src->audiocast_udpdata_mutex);
+          g_free (src->iradio_title);
+          src->iradio_title = value;
+          g_mutex_unlock (src->audiocast_udpdata_mutex);
+
+          g_mutex_lock (src->audiocast_queue_mutex);
+          src->audiocast_notify_queue =
+              g_list_append (src->audiocast_notify_queue, "iradio-title");
+          GST_DEBUG_OBJECT (src, "audiocast title: %s\n", src->iradio_title);
+          g_mutex_unlock (src->audiocast_queue_mutex);
+        } else if (!strncmp (lines[i], "x-audiocast-streamurl", 21)) {
+          g_mutex_lock (src->audiocast_udpdata_mutex);
+          g_free (src->iradio_url);
+          src->iradio_url = value;
+          g_mutex_unlock (src->audiocast_udpdata_mutex);
+
+          g_mutex_lock (src->audiocast_queue_mutex);
+          src->audiocast_notify_queue =
+              g_list_append (src->audiocast_notify_queue, "iradio-url");
+          GST_DEBUG_OBJECT (src, "audiocast url: %s\n", src->iradio_title);
+          g_mutex_unlock (src->audiocast_queue_mutex);
+        } else if (!strncmp (lines[i], "x-audiocast-udpseqnr", 20)) {
+          gchar outbuf[120];
+
+          sprintf (outbuf, "x-audiocast-ack: %ld \r\n", atol (value));
+          g_free (value);
+
+          if (sendto (src->audiocast_fd, outbuf, strlen (outbuf), 0,
+                  (struct sockaddr *) &from, fromlen) <= 0) {
+            g_print ("Error sending response to server: %s\n",
+                strerror (errno));
+            continue;
+          }
+          GST_DEBUG_OBJECT (src, "sent audiocast ack: %s\n", outbuf);
+        }
+      }
+      g_strfreev (lines);
+    }
+  }
+  return NULL;
+}
+
+static void
+audiocast_thread_kill (GstGnomeVFSSrc * src)
+{
+  if (!src->audiocast_thread)
+    return;
+
+  /*
+     We rely on this hack to kill the
+     audiocast thread.  If we get icecast
+     metadata, then we don't need the
+     audiocast metadata too.
+   */
+  GST_DEBUG ("audiocast: writing die character");
+  if (write (src->audiocast_thread_die_outfd, "q", 1) != 1)
+    g_critical ("gnomevfssrc: could not write to audiocast thread fd");
+  close (src->audiocast_thread_die_outfd);
+  GST_DEBUG ("audiocast: joining thread");
+  g_thread_join (src->audiocast_thread);
+  src->audiocast_thread = NULL;
 }
 
 static void
@@ -482,6 +751,8 @@ gst_gnome_vfs_src_send_additional_headers_callback (gconstpointer in,
 
   out_args->headers = g_list_append (out_args->headers,
       g_strdup ("icy-metadata:1\r\n"));
+  out_args->headers = g_list_append (out_args->headers,
+      g_strdup_printf ("x-audiocast-udpport: %d\r\n", src->audiocast_port));
 }
 
 static void
@@ -498,13 +769,6 @@ gst_gnome_vfs_src_received_headers_callback (gconstpointer in,
   if (!src->iradio_mode)
     return;
 
-  GST_DEBUG_OBJECT (src, "receiving internet radio metadata\n");
-
-  /* FIXME: Could we use "Accept-Ranges: bytes"
-   * http://www.w3.org/Protocols/rfc2616/rfc2616-sec14.html#sec14.5
-   * to enable pull-mode?
-   */
-
   for (i = in_args->headers; i; i = i->next) {
     char *data = (char *) i->data;
     char *key = data;
@@ -518,25 +782,21 @@ gst_gnome_vfs_src_received_headers_callback (gconstpointer in,
     if (!strlen (value))
       continue;
 
-    GST_LOG_OBJECT (src, "data %s", data);
-
     /* Icecast stuff */
     if (strncmp (data, "icy-metaint:", 12) == 0) {      /* ugh */
       if (sscanf (data + 12, "%d", &icy_metaint) == 1) {
-        if (icy_metaint > 0) {
-          GstCaps *icy_caps;
-
-          icy_caps = gst_caps_new_simple ("application/x-icy",
-              "metadata-interval", G_TYPE_INT, icy_metaint, NULL);
-          gst_pad_set_caps (GST_BASE_SRC_PAD (src), icy_caps);
-          gst_caps_unref (icy_caps);
-        }
+        src->icy_metaint = icy_metaint;
+        GST_DEBUG_OBJECT (src, "got icy-metaint %d, killing audiocast thread",
+            src->icy_metaint);
+        audiocast_thread_kill (src);
+        continue;
       }
-      continue;
     }
 
     if (!strncmp (data, "icy-", 4))
       key = data + 4;
+    else if (!strncmp (data, "x-audiocast-", 12))
+      key = data + 12;
     else
       continue;
 
@@ -588,10 +848,80 @@ gst_gnome_vfs_src_pop_callbacks (GstGnomeVFSSrc * src)
       (GNOME_VFS_MODULE_CALLBACK_HTTP_SEND_ADDITIONAL_HEADERS);
   gnome_vfs_module_callback_pop
       (GNOME_VFS_MODULE_CALLBACK_HTTP_RECEIVED_HEADERS);
-
-  src->http_callbacks_pushed = FALSE;
 }
 
+static void
+gst_gnome_vfs_src_get_icy_metadata (GstGnomeVFSSrc * src)
+{
+  GnomeVFSFileSize length = 0;
+  GnomeVFSResult res;
+  gint metadata_length;
+  guchar foobyte;
+  guchar *data;
+  guchar *pos;
+  gchar **tags;
+  int i;
+
+  GST_DEBUG_OBJECT (src, "reading icecast metadata");
+
+  while (length == 0) {
+    res = gnome_vfs_read (src->handle, &foobyte, 1, &length);
+    if (res != GNOME_VFS_OK)
+      return;
+  }
+
+  metadata_length = foobyte * 16;
+
+  if (metadata_length == 0)
+    return;
+
+  data = g_new (guchar, metadata_length + 1);
+  pos = data;
+
+  while (pos - data < metadata_length) {
+    res = gnome_vfs_read (src->handle, pos,
+        metadata_length - (pos - data), &length);
+    /* FIXME: better error handling here? */
+    if (res != GNOME_VFS_OK) {
+      g_free (data);
+      return;
+    }
+
+    pos += length;
+  }
+
+  data[metadata_length] = 0;
+  tags = g_strsplit ((gchar *) data, "';", 0);
+
+  for (i = 0; tags[i]; i++) {
+    if (!g_ascii_strncasecmp (tags[i], "StreamTitle=", 12)) {
+      g_free (src->iradio_title);
+      src->iradio_title = gst_gnome_vfs_src_unicodify (tags[i] + 13);
+      if (src->iradio_title) {
+        GST_DEBUG_OBJECT (src, "sending notification on icecast title");
+        g_object_notify (G_OBJECT (src), "iradio-title");
+      } else
+        g_print ("Unable to convert icecast title \"%s\" to UTF-8!\n",
+            tags[i] + 13);
+
+    }
+    if (!g_ascii_strncasecmp (tags[i], "StreamUrl=", 10)) {
+      g_free (src->iradio_url);
+      src->iradio_url = gst_gnome_vfs_src_unicodify (tags[i] + 11);
+      if (src->iradio_url) {
+        GST_DEBUG_OBJECT (src, "sending notification on icecast url");
+        g_object_notify (G_OBJECT (src), "iradio-url");
+      } else
+        g_print ("Unable to convert icecast url \"%s\" to UTF-8!\n",
+            tags[i] + 11);
+    }
+  }
+
+  g_strfreev (tags);
+}
+
+/* end of icecast/audiocast metadata extraction support code */
+
 /*
  * Read a new buffer from src->reqoffset, takes care of events
  * and seeking and such.
@@ -604,19 +934,18 @@ gst_gnome_vfs_src_create (GstBaseSrc * basesrc, guint64 offset, guint size,
   GstBuffer *buf;
   GnomeVFSFileSize readbytes;
   guint8 *data;
-  guint todo;
   GstGnomeVFSSrc *src;
 
   src = GST_GNOME_VFS_SRC (basesrc);
 
-  GST_DEBUG ("now at %" G_GINT64_FORMAT ", reading from %" G_GUINT64_FORMAT
-      ", size %u", src->curoffset, offset, size);
+  GST_DEBUG ("now at %llu, reading %lld, size %u", src->curoffset, offset,
+      size);
 
   /* seek if required */
-  if (G_UNLIKELY (src->curoffset != offset)) {
+  if (src->curoffset != offset) {
     GST_DEBUG ("need to seek");
     if (src->seekable) {
-      GST_DEBUG ("seeking to %" G_GUINT64_FORMAT, offset);
+      GST_DEBUG ("seeking to %lld", offset);
       res = gnome_vfs_seek (src->handle, GNOME_VFS_SEEK_START, offset);
       if (res != GNOME_VFS_OK)
         goto seek_failed;
@@ -626,36 +955,53 @@ gst_gnome_vfs_src_create (GstBaseSrc * basesrc, guint64 offset, guint size,
     }
   }
 
-  buf = gst_buffer_try_new_and_alloc (size);
-  if (G_UNLIKELY (buf == NULL && size == 0)) {
-    GST_ERROR_OBJECT (src, "Failed to allocate %u bytes", size);
-    return GST_FLOW_ERROR;
-  }
+  audiocast_do_notifications (src);
 
-  data = GST_BUFFER_DATA (buf);
+  if (src->iradio_mode && src->icy_metaint > 0) {
+    buf = gst_buffer_new_and_alloc (src->icy_metaint);
 
-  todo = size;
-  while (todo > 0) {
-    /* this can return less that we ask for */
-    res = gnome_vfs_read (src->handle, data, todo, &readbytes);
+    data = GST_BUFFER_DATA (buf);
 
-    if (G_UNLIKELY (res == GNOME_VFS_ERROR_EOF || (res == GNOME_VFS_OK
-                && readbytes == 0)))
+    /* try to read */
+    GST_DEBUG_OBJECT (src, "doing read: icy_count: %" G_GINT64_FORMAT,
+        src->icy_count);
+
+    res = gnome_vfs_read (src->handle, data,
+        src->icy_metaint - src->icy_count, &readbytes);
+
+    if (res == GNOME_VFS_ERROR_EOF || (res == GNOME_VFS_OK && readbytes == 0))
       goto eos;
 
-    if (G_UNLIKELY (res != GNOME_VFS_OK))
+    if (res != GNOME_VFS_OK)
       goto read_failed;
 
-    if (readbytes < todo) {
-      data = &data[readbytes];
-      todo -= readbytes;
-    } else {
-      todo = 0;
+    src->icy_count += readbytes;
+    GST_BUFFER_OFFSET (buf) = src->curoffset;
+    GST_BUFFER_SIZE (buf) = readbytes;
+    src->curoffset += readbytes;
+
+    if (src->icy_count == src->icy_metaint) {
+      gst_gnome_vfs_src_get_icy_metadata (src);
+      src->icy_count = 0;
     }
-    GST_LOG ("  got size %" G_GUINT64_FORMAT, readbytes);
+  } else {
+    buf = gst_buffer_new_and_alloc (size);
+
+    data = GST_BUFFER_DATA (buf);
+    GST_BUFFER_OFFSET (buf) = src->curoffset;
+
+    res = gnome_vfs_read (src->handle, data, size, &readbytes);
+
+    if (res == GNOME_VFS_ERROR_EOF || (res == GNOME_VFS_OK && readbytes == 0))
+      goto eos;
+
+    GST_BUFFER_SIZE (buf) = readbytes;
+
+    if (res != GNOME_VFS_OK)
+      goto read_failed;
+
+    src->curoffset += readbytes;
   }
-  GST_BUFFER_OFFSET (buf) = src->curoffset;
-  src->curoffset += size;
 
   /* we're done, return the buffer */
   *buffer = buf;
@@ -673,7 +1019,7 @@ cannot_seek:
   {
     GST_ELEMENT_ERROR (src, RESOURCE, SEEK, (NULL),
         ("Requested seek from %" G_GINT64_FORMAT " to %" G_GINT64_FORMAT
-            " on non-seekable stream", src->curoffset, offset));
+            "on non-seekable stream", src->curoffset, offset));
     return GST_FLOW_ERROR;
   }
 read_failed:
@@ -687,32 +1033,11 @@ eos:
   {
     gst_buffer_unref (buf);
     GST_DEBUG_OBJECT (src, "Reading data gave EOS");
+    gst_pad_push_event (basesrc->srcpad, gst_event_new_eos ());
     return GST_FLOW_UNEXPECTED;
   }
 }
 
-static gboolean
-gst_gnome_vfs_src_query (GstBaseSrc * basesrc, GstQuery * query)
-{
-  gboolean ret = FALSE;
-  GstGnomeVFSSrc *src = GST_GNOME_VFS_SRC (basesrc);
-
-  switch (GST_QUERY_TYPE (query)) {
-    case GST_QUERY_URI:
-      gst_query_set_uri (query, src->uri_name);
-      ret = TRUE;
-      break;
-    default:
-      ret = FALSE;
-      break;
-  }
-
-  if (!ret)
-    ret = GST_BASE_SRC_CLASS (parent_class)->query (basesrc, query);
-
-  return ret;
-}
-
 static gboolean
 gst_gnome_vfs_src_is_seekable (GstBaseSrc * basesrc)
 {
@@ -723,153 +1048,93 @@ gst_gnome_vfs_src_is_seekable (GstBaseSrc * basesrc)
   return src->seekable;
 }
 
-static gboolean
-gst_gnome_vfs_src_check_get_range (GstBaseSrc * basesrc)
-{
-  GstGnomeVFSSrc *src;
-  const gchar *protocol;
-
-  src = GST_GNOME_VFS_SRC (basesrc);
-
-  if (src->uri == NULL) {
-    GST_WARNING_OBJECT (src, "no URI set yet");
-    return FALSE;
-  }
-
-  if (gnome_vfs_uri_is_local (src->uri)) {
-    GST_LOG_OBJECT (src, "local URI (%s), assuming random access is possible",
-        GST_STR_NULL (src->uri_name));
-    return TRUE;
-  }
-
-  /* blacklist certain protocols we know won't work getrange-based */
-  protocol = gnome_vfs_uri_get_scheme (src->uri);
-  if (protocol == NULL)
-    goto undecided;
-
-  if (strcmp (protocol, "http") == 0 || strcmp (protocol, "https") == 0) {
-    GST_LOG_OBJECT (src, "blacklisted protocol '%s', no random access possible"
-        " (URI=%s)", protocol, GST_STR_NULL (src->uri_name));
-    return FALSE;
-  }
-
-  /* fall through to undecided */
-
-undecided:
-  {
-    /* don't know what to do, let the basesrc class decide for us */
-    GST_LOG_OBJECT (src, "undecided about URI '%s', let base class handle it",
-        GST_STR_NULL (src->uri_name));
-
-    if (GST_BASE_SRC_CLASS (parent_class)->check_get_range)
-      return GST_BASE_SRC_CLASS (parent_class)->check_get_range (basesrc);
-
-    return FALSE;
-  }
-}
-
 static gboolean
 gst_gnome_vfs_src_get_size (GstBaseSrc * basesrc, guint64 * size)
 {
   GstGnomeVFSSrc *src;
-  GnomeVFSFileInfo *info;
-  GnomeVFSFileInfoOptions options;
-  GnomeVFSResult res;
 
   src = GST_GNOME_VFS_SRC (basesrc);
 
-  *size = -1;
-  info = gnome_vfs_file_info_new ();
-  options = GNOME_VFS_FILE_INFO_DEFAULT | GNOME_VFS_FILE_INFO_FOLLOW_LINKS;
-  res = gnome_vfs_get_file_info_from_handle (src->handle, info, options);
-  if (res == GNOME_VFS_OK) {
-    if ((info->valid_fields & GNOME_VFS_FILE_INFO_FIELDS_SIZE) != 0) {
-      *size = info->size;
-      GST_DEBUG_OBJECT (src, "from handle: %" G_GUINT64_FORMAT " bytes", *size);
-    } else if (src->own_handle && gnome_vfs_uri_is_local (src->uri)) {
-      GST_DEBUG_OBJECT (src,
-          "file size not known, file local, trying fallback");
-      res = gnome_vfs_get_file_info_uri (src->uri, info, options);
-      if (res == GNOME_VFS_OK &&
-          (info->valid_fields & GNOME_VFS_FILE_INFO_FIELDS_SIZE) != 0) {
-        *size = info->size;
-        GST_DEBUG_OBJECT (src, "from uri: %" G_GUINT64_FORMAT " bytes", *size);
+  GST_DEBUG_OBJECT (src, "size %" G_GUINT64_FORMAT, src->size);
+
+  if (src->size == (GnomeVFSFileSize) - 1)
+    return FALSE;
+
+  *size = src->size;
+
+  return TRUE;
+}
+
+/* open the file, do stuff necessary to go to READY state */
+static gboolean
+gst_gnome_vfs_src_start (GstBaseSrc * basesrc)
+{
+  GnomeVFSResult res;
+  GnomeVFSFileInfo *info;
+  GstGnomeVFSSrc *src;
+
+  src = GST_GNOME_VFS_SRC (basesrc);
+
+  if (!audiocast_init (src))
+    return FALSE;
+
+  gst_gnome_vfs_src_push_callbacks (src);
+
+  if (src->uri != NULL) {
+    if ((res = gnome_vfs_open_uri (&src->handle, src->uri,
+                GNOME_VFS_OPEN_READ)) != GNOME_VFS_OK) {
+      gchar *filename = gnome_vfs_uri_to_string (src->uri,
+          GNOME_VFS_URI_HIDE_PASSWORD);
+
+      gst_gnome_vfs_src_pop_callbacks (src);
+      audiocast_thread_kill (src);
+
+      if (res == GNOME_VFS_ERROR_NOT_FOUND ||
+          res == GNOME_VFS_ERROR_SERVICE_NOT_AVAILABLE) {
+        GST_ELEMENT_ERROR (src, RESOURCE, NOT_FOUND, (NULL),
+            ("Could not open vfs file \"%s\" for reading: %s",
+                filename, gnome_vfs_result_to_string (res)));
+      } else {
+        GST_ELEMENT_ERROR (src, RESOURCE, OPEN_READ, (NULL),
+            ("Could not open vfs file \"%s\" for reading: %s",
+                filename, gnome_vfs_result_to_string (res)));
       }
+      g_free (filename);
+      return FALSE;
     }
+    src->own_handle = TRUE;
+  } else if (!src->handle) {
+    GST_ELEMENT_ERROR (src, RESOURCE, OPEN_READ, (NULL), ("No filename given"));
+    return FALSE;
+  } else {
+    src->own_handle = FALSE;
+  }
+
+  src->size = (GnomeVFSFileSize) - 1;
+  info = gnome_vfs_file_info_new ();
+  if ((res = gnome_vfs_get_file_info_from_handle (src->handle,
+              info, GNOME_VFS_FILE_INFO_DEFAULT)) == GNOME_VFS_OK) {
+    if (info->valid_fields & GNOME_VFS_FILE_INFO_FIELDS_SIZE) {
+      src->size = info->size;
+      GST_DEBUG_OBJECT (src, "size: %llu bytes", src->size);
+    } else
+      GST_LOG_OBJECT (src, "filesize not known");
   } else {
     GST_WARNING_OBJECT (src, "getting info failed: %s",
         gnome_vfs_result_to_string (res));
   }
   gnome_vfs_file_info_unref (info);
 
-  if (*size == (GnomeVFSFileSize) - 1)
-    return FALSE;
+  audiocast_do_notifications (src);
 
-  GST_DEBUG_OBJECT (src, "return size %" G_GUINT64_FORMAT, *size);
-
-  return TRUE;
-}
-
-/* open the file, do stuff necessary to go to PAUSED state */
-static gboolean
-gst_gnome_vfs_src_start (GstBaseSrc * basesrc)
-{
-  GnomeVFSResult res;
-  GstGnomeVFSSrc *src;
-
-  src = GST_GNOME_VFS_SRC (basesrc);
-
-  gst_gnome_vfs_src_push_callbacks (src);
-
-  if (src->uri != NULL) {
-    GnomeVFSOpenMode mode = GNOME_VFS_OPEN_READ;
-
-    /* this can block... */
-    res = gnome_vfs_open_uri (&src->handle, src->uri, mode);
-    if (res != GNOME_VFS_OK)
-      goto open_failed;
-    src->own_handle = TRUE;
-  } else if (!src->handle) {
-    goto no_filename;
-  } else {
-    src->own_handle = FALSE;
-  }
-
-  if (gnome_vfs_seek (src->handle, GNOME_VFS_SEEK_CURRENT, 0) == GNOME_VFS_OK) {
+  if (gnome_vfs_seek (src->handle, GNOME_VFS_SEEK_CURRENT, 0)
+      == GNOME_VFS_OK) {
     src->seekable = TRUE;
   } else {
     src->seekable = FALSE;
   }
 
   return TRUE;
-
-  /* ERRORS */
-open_failed:
-  {
-    gchar *filename = gnome_vfs_uri_to_string (src->uri,
-        GNOME_VFS_URI_HIDE_PASSWORD);
-
-    gst_gnome_vfs_src_pop_callbacks (src);
-
-    if (res == GNOME_VFS_ERROR_NOT_FOUND ||
-        res == GNOME_VFS_ERROR_HOST_NOT_FOUND ||
-        res == GNOME_VFS_ERROR_SERVICE_NOT_AVAILABLE) {
-      GST_ELEMENT_ERROR (src, RESOURCE, NOT_FOUND, (NULL),
-          ("Could not open vfs file \"%s\" for reading: %s (%d)",
-              filename, gnome_vfs_result_to_string (res), res));
-    } else {
-      GST_ELEMENT_ERROR (src, RESOURCE, OPEN_READ, (NULL),
-          ("Could not open vfs file \"%s\" for reading: %s (%d)",
-              filename, gnome_vfs_result_to_string (res), res));
-    }
-    g_free (filename);
-    return FALSE;
-  }
-no_filename:
-  {
-    GST_ELEMENT_ERROR (src, RESOURCE, OPEN_READ, (NULL), ("No filename given"));
-    return FALSE;
-  }
 }
 
 static gboolean
@@ -880,17 +1145,13 @@ gst_gnome_vfs_src_stop (GstBaseSrc * basesrc)
   src = GST_GNOME_VFS_SRC (basesrc);
 
   gst_gnome_vfs_src_pop_callbacks (src);
+  audiocast_thread_kill (src);
 
   if (src->own_handle) {
-    GnomeVFSResult res;
-
-    res = gnome_vfs_close (src->handle);
-    if (res != GNOME_VFS_OK) {
-      GST_ELEMENT_ERROR (src, RESOURCE, CLOSE, (NULL),
-          ("Could not close vfs handle: %s", gnome_vfs_result_to_string (res)));
-    }
+    gnome_vfs_close (src->handle);
     src->handle = NULL;
   }
+  src->size = (GnomeVFSFileSize) - 1;
   src->curoffset = 0;
 
   return TRUE;
