@@ -339,7 +339,7 @@ static const GstQueryType *
 gst_ogg_pad_query_types (GstPad * pad)
 {
   static const GstQueryType query_types[] = {
-    GST_QUERY_POSITION,
+    GST_QUERY_DURATION,
     0
   };
 
@@ -378,12 +378,8 @@ gst_ogg_pad_src_query (GstPad * pad, GstQuery * query)
       gst_query_set_duration (query, GST_FORMAT_TIME, ogg->total_time);
       break;
     }
-    case GST_QUERY_CONVERT:
-      /* hmm .. */
-      res = FALSE;
-      break;
     default:
-      res = FALSE;
+      res = gst_pad_query_default (pad, query);
       break;
   }
 done:
@@ -624,7 +620,10 @@ gst_ogg_pad_typefind (GstOggPad * pad, ogg_packet * packet)
 {
   GstCaps *caps;
   GstElement *element = NULL;
+
+#ifndef GST_DISABLE_GST_DEBUG
   GstOggDemux *ogg = pad->ogg;
+#endif
 
   if (GST_PAD_CAPS (pad) != NULL)
     return TRUE;
@@ -703,7 +702,10 @@ gst_ogg_demux_chain_elem_pad (GstOggPad * pad, ogg_packet * packet)
 {
   GstBuffer *buf;
   GstFlowReturn ret;
+
+#ifndef GST_DISABLE_GST_DEBUG
   GstOggDemux *ogg = pad->ogg;
+#endif
 
   /* initialize our internal decoder with packets */
   if (!pad->elem_pad)
@@ -741,7 +743,10 @@ static GstFlowReturn
 gst_ogg_demux_queue_data (GstOggPad * pad, ogg_packet * packet)
 {
   GstBuffer *buf;
+
+#ifndef GST_DISABLE_GST_DEBUG
   GstOggDemux *ogg = pad->ogg;
+#endif
 
   GST_DEBUG_OBJECT (ogg, "%p queueing data serial %08lx", pad, pad->serialno);
 
@@ -2709,6 +2714,12 @@ gst_ogg_demux_plugin_init (GstPlugin * plugin)
   GST_DEBUG_CATEGORY_INIT (gst_ogg_demux_setup_debug, "oggdemux_setup", 0,
       "ogg demuxer setup stage when parsing pipeline");
 
+#if ENABLE_NLS
+  GST_DEBUG ("binding text domain %s to locale dir %s", GETTEXT_PACKAGE,
+      LOCALEDIR);
+  bindtextdomain (GETTEXT_PACKAGE, LOCALEDIR);
+#endif
+
   return gst_element_register (plugin, "oggdemux", GST_RANK_PRIMARY,
       GST_TYPE_OGG_DEMUX);
 }
