@@ -39,6 +39,14 @@ GST_DEBUG_CATEGORY_STATIC (gst_ogg_mux_debug);
 #define GST_IS_OGG_MUX(obj) (G_TYPE_CHECK_INSTANCE_TYPE((obj),GST_TYPE_OGG_MUX))
 #define GST_IS_OGG_MUX_CLASS(obj) (G_TYPE_CHECK_CLASS_TYPE((klass),GST_TYPE_OGG_MUX))
 
+/* This isn't generally what you'd want with an end-time macro, because
+   technically the end time of a buffer with invalid duration is invalid. But
+   for sorting ogg pages this is what we want. */
+#define GST_BUFFER_END_TIME(buf) \
+    (GST_BUFFER_DURATION_IS_VALID (buf) \
+    ? GST_BUFFER_TIMESTAMP (buf) + GST_BUFFER_DURATION (buf) \
+    : GST_BUFFER_TIMESTAMP (buf))
+
 #define GST_GP_FORMAT "[gp %8" G_GINT64_FORMAT "]"
 
 typedef struct _GstOggMux GstOggMux;
@@ -157,7 +165,7 @@ static GstStaticPadTemplate sink_factory = GST_STATIC_PAD_TEMPLATE ("sink_%d",
     GST_PAD_REQUEST,
     GST_STATIC_CAPS ("video/x-theora; "
         "audio/x-vorbis; audio/x-flac; audio/x-speex; "
-        "application/x-ogm-video; application/x-ogm-audio")
+        "application/x-ogm-video; application/x-ogm-audio; video/x-dirac")
     );
 
 static void gst_ogg_mux_base_init (gpointer g_class);
@@ -170,6 +178,8 @@ gst_ogg_mux_collected (GstCollectPads * pads, GstOggMux * ogg_mux);
 static gboolean gst_ogg_mux_handle_src_event (GstPad * pad, GstEvent * event);
 static GstPad *gst_ogg_mux_request_new_pad (GstElement * element,
     GstPadTemplate * templ, const gchar * name);
+static void gst_ogg_mux_release_pad (GstElement * element, GstPad * pad);
+
 static void gst_ogg_mux_set_property (GObject * object,
     guint prop_id, const GValue * value, GParamSpec * pspec);
 static void gst_ogg_mux_get_property (GObject * object,
@@ -235,6 +245,7 @@ gst_ogg_mux_class_init (GstOggMuxClass * klass)
   gobject_class->set_property = gst_ogg_mux_set_property;
 
   gstelement_class->request_new_pad = gst_ogg_mux_request_new_pad;
+  gstelement_class->release_pad = gst_ogg_mux_release_pad;
 
   g_object_class_install_property (gobject_class, ARG_MAX_DELAY,
       g_param_spec_uint64 ("max-delay", "Max delay",
@@ -384,10 +395,6 @@ gst_ogg_mux_request_new_pad (GstElement * element,
       oggpad->new_page = TRUE;
       oggpad->first_delta = FALSE;
       oggpad->prev_delta = FALSE;
-      /* TODO: delete this queue (and the things contained within) later,
-       * possibly when doing gst_collect_pads_remove_pad() (which we don't seem
-       * to do at all?)
-       */
       oggpad->pagebuffers = g_queue_new ();
     }
   }
@@ -412,15 +419,50 @@ wrong_template:
   }
 }
 
+static void
+gst_ogg_mux_release_pad (GstElement * element, GstPad * pad)
+{
+  GstOggMux *ogg_mux;
+  GSList *walk;
+
+  ogg_mux = GST_OGG_MUX (gst_pad_get_parent (pad));
+
+  /* FIXME: When a request pad is released while paused or playing, 
+   * we probably need to do something to finalise its stream in the
+   * ogg data we're producing, but I'm not sure what */
+
+  /* Find out GstOggPad in the collect pads info and clean it up */
+
+  GST_OBJECT_LOCK (ogg_mux->collect);
+  walk = ogg_mux->collect->data;
+  while (walk) {
+    GstOggPad *oggpad = (GstOggPad *) walk->data;
+    GstCollectData *cdata = (GstCollectData *) walk->data;
+    GstBuffer *buf;
+
+    if (cdata->pad == pad) {
+      /* FIXME: clear the ogg stream stuff? - 
+       *    ogg_stream_clear (&oggpad->stream); */
+
+      while ((buf = g_queue_pop_head (oggpad->pagebuffers)) != NULL) {
+        gst_buffer_unref (buf);
+      }
+
+      g_queue_free (oggpad->pagebuffers);
+    }
+    walk = g_slist_next (walk);
+  }
+  GST_OBJECT_UNLOCK (ogg_mux->collect);
+
+  gst_collect_pads_remove_pad (ogg_mux->collect, pad);
+}
+
 /* handle events */
 static gboolean
 gst_ogg_mux_handle_src_event (GstPad * pad, GstEvent * event)
 {
-  GstOggMux *ogg_mux;
   GstEventType type;
 
-  ogg_mux = GST_OGG_MUX (gst_pad_get_parent (pad));
-
   type = event ? GST_EVENT_TYPE (event) : GST_EVENT_UNKNOWN;
 
   switch (type) {
@@ -537,12 +579,12 @@ gst_ogg_mux_dequeue_page (GstOggMux * mux, GstFlowReturn * flowret)
     if (buf) {
       /* if no oldest buffer yet, take this one */
       if (oldest == GST_CLOCK_TIME_NONE) {
-        oldest = GST_BUFFER_TIMESTAMP (buf);
+        oldest = GST_BUFFER_END_TIME (buf);
         opad = pad;
       } else {
         /* if we have an oldest, compare with this one */
-        if (GST_BUFFER_TIMESTAMP (buf) < oldest) {
-          oldest = GST_BUFFER_TIMESTAMP (buf);
+        if (GST_BUFFER_END_TIME (buf) < oldest) {
+          oldest = GST_BUFFER_END_TIME (buf);
           opad = pad;
         }
       }
@@ -554,9 +596,8 @@ gst_ogg_mux_dequeue_page (GstOggMux * mux, GstFlowReturn * flowret)
     g_assert (opad);
     buf = g_queue_pop_head (opad->pagebuffers);
     GST_LOG_OBJECT (opad,
-        GST_GP_FORMAT " pushing oldest page (time %" GST_TIME_FORMAT ")",
-        GST_BUFFER_OFFSET_END (buf),
-        GST_TIME_ARGS (GST_BUFFER_TIMESTAMP (buf)));
+        GST_GP_FORMAT " pushing oldest page (end time %" GST_TIME_FORMAT ")",
+        GST_BUFFER_OFFSET_END (buf), GST_TIME_ARGS (GST_BUFFER_END_TIME (buf)));
     *flowret = gst_ogg_mux_push_buffer (mux, buf);
     ret = TRUE;
   }
@@ -1299,6 +1340,7 @@ gst_ogg_mux_set_property (GObject * object,
   }
 }
 
+/* Clear all buffers from the collectpads object */
 static void
 gst_ogg_mux_clear_collectpads (GstCollectPads * collect)
 {
@@ -1306,18 +1348,27 @@ gst_ogg_mux_clear_collectpads (GstCollectPads * collect)
 
   walk = collect->data;
   while (walk) {
-    GstOggPad *pad = (GstOggPad *) walk->data;
+    GstOggPad *oggpad = (GstOggPad *) walk->data;
     GstBuffer *buf;
 
-    ogg_stream_clear (&pad->stream);
+    ogg_stream_clear (&oggpad->stream);
 
-    while ((buf = g_queue_pop_head (pad->pagebuffers)) != NULL) {
+    while ((buf = g_queue_pop_head (oggpad->pagebuffers)) != NULL) {
       gst_buffer_unref (buf);
     }
-    g_queue_free (pad->pagebuffers);
 
-    gst_collect_pads_remove_pad (collect, ((GstCollectData *) pad)->pad);
-    walk = collect->data;
+    ogg_stream_init (&oggpad->stream, oggpad->serial);
+    oggpad->packetno = 0;
+    oggpad->pageno = 0;
+    oggpad->eos = FALSE;
+    /* we assume there will be some control data first for this pad */
+    oggpad->state = GST_OGG_PAD_STATE_CONTROL;
+    oggpad->new_page = TRUE;
+    oggpad->first_delta = FALSE;
+    oggpad->prev_delta = FALSE;
+    oggpad->pagebuffers = g_queue_new ();
+
+    walk = g_slist_next (walk);
   }
 }
 
