@@ -36,12 +36,6 @@
 #include <ogg/ogg.h>
 #include <string.h>
 
-static GstElementDetails gst_ogg_parse_details =
-GST_ELEMENT_DETAILS ("Ogg parser",
-    "Codec/Parser",
-    "parse ogg streams into pages (info about ogg: http://xiph.org)",
-    "Michael Smith <msmith@fluendo.com>");
-
 GST_DEBUG_CATEGORY_STATIC (gst_ogg_parse_debug);
 #define GST_CAT_DEFAULT gst_ogg_parse_debug
 
@@ -170,10 +164,10 @@ gst_ogg_parse_new_stream (GstOggParse * parser, guint32 serialno)
 static GstOggStream *
 gst_ogg_parse_find_stream (GstOggParse * parser, guint32 serialno)
 {
-  GSList *l;
+  gint i;
 
-  for (l = parser->oggstreams; l != NULL; l = l->next) {
-    GstOggStream *stream = (GstOggStream *) l->data;
+  for (i = 0; i < g_slist_length (parser->oggstreams); i++) {
+    GstOggStream *stream = g_slist_nth_data (parser->oggstreams, i);
 
     if (stream->serialno == serialno)
       return stream;
@@ -217,6 +211,11 @@ static void
 gst_ogg_parse_base_init (gpointer g_class)
 {
   GstElementClass *element_class = GST_ELEMENT_CLASS (g_class);
+  static GstElementDetails gst_ogg_parse_details =
+      GST_ELEMENT_DETAILS ("ogg parser",
+      "Codec/Parser",
+      "parse ogg streams into pages (info about ogg: http://xiph.org)",
+      "Michael Smith <msmith@fluendo.com>");
 
   gst_element_class_set_details (element_class, &gst_ogg_parse_details);
 
@@ -232,7 +231,7 @@ gst_ogg_parse_class_init (GstOggParseClass * klass)
   GstElementClass *gstelement_class = GST_ELEMENT_CLASS (klass);
   GObjectClass *gobject_class = G_OBJECT_CLASS (klass);
 
-  parent_class = g_type_class_peek_parent (klass);
+  parent_class = g_type_class_ref (GST_TYPE_ELEMENT);
 
   gstelement_class->change_state = gst_ogg_parse_change_state;
 
@@ -505,14 +504,13 @@ gst_ogg_parse_chain (GstPad * pad, GstBuffer * buffer)
             GstCaps *caps;
             GstStructure *structure;
             GValue array = { 0 };
-            gint count = 0;
+            gint i, count = 0;
             gboolean found_pending_headers = FALSE;
-            GSList *l;
 
             g_value_init (&array, GST_TYPE_ARRAY);
 
-            for (l = ogg->oggstreams; l != NULL; l = l->next) {
-              GstOggStream *stream = (GstOggStream *) l->data;
+            for (i = 0; i < g_slist_length (ogg->oggstreams); i++) {
+              GstOggStream *stream = g_slist_nth_data (ogg->oggstreams, i);
 
               if (g_slist_length (stream->headers) == 0) {
                 GST_LOG_OBJECT (ogg, "No primary header found for stream %u",
@@ -525,8 +523,8 @@ gst_ogg_parse_chain (GstPad * pad, GstBuffer * buffer)
               count++;
             }
 
-            for (l = ogg->oggstreams; l != NULL; l = l->next) {
-              GstOggStream *stream = (GstOggStream *) l->data;
+            for (i = 0; i < g_slist_length (ogg->oggstreams); i++) {
+              GstOggStream *stream = g_slist_nth_data (ogg->oggstreams, i);
               int j;
 
               for (j = 1; j < g_slist_length (stream->headers); j++) {
@@ -552,16 +550,16 @@ gst_ogg_parse_chain (GstPad * pad, GstBuffer * buffer)
                 "(one per page)", count);
 
             /* Now, we do the same thing, but push buffers... */
-            for (l = ogg->oggstreams; l != NULL; l = l->next) {
-              GstOggStream *stream = (GstOggStream *) l->data;
+            for (i = 0; i < g_slist_length (ogg->oggstreams); i++) {
+              GstOggStream *stream = g_slist_nth_data (ogg->oggstreams, i);
 
               result = gst_pad_push (ogg->srcpad,
                   GST_BUFFER (stream->headers->data));
               if (result != GST_FLOW_OK)
                 return result;
             }
-            for (l = ogg->oggstreams; l != NULL; l = l->next) {
-              GstOggStream *stream = (GstOggStream *) l->data;
+            for (i = 0; i < g_slist_length (ogg->oggstreams); i++) {
+              GstOggStream *stream = g_slist_nth_data (ogg->oggstreams, i);
               int j;
 
               for (j = 1; j < g_slist_length (stream->headers); j++) {
@@ -575,27 +573,24 @@ gst_ogg_parse_chain (GstPad * pad, GstBuffer * buffer)
             ogg->in_headers = 0;
 
             /* And finally the pending data pages */
-            for (l = ogg->oggstreams; l != NULL; l = l->next) {
-              GstOggStream *stream = (GstOggStream *) l->data;
-              GSList *k;
+            for (i = 0; i < g_slist_length (ogg->oggstreams); i++) {
+              GstOggStream *stream = g_slist_nth_data (ogg->oggstreams, i);
+              int j;
 
-              if (stream->unknown_pages == NULL)
+              if (g_slist_length (stream->unknown_pages) > 0) {
+                if (found_pending_headers) {
+                  GST_WARNING_OBJECT (ogg, "Incorrectly muxed headers found at "
+                      "approximate offset %lld", ogg->offset);
+                }
+                found_pending_headers = TRUE;
+              } else
                 continue;
 
-              if (found_pending_headers) {
-                GST_WARNING_OBJECT (ogg, "Incorrectly muxed headers found at "
-                    "approximate offset %lld", ogg->offset);
-              }
-              found_pending_headers = TRUE;
-
               GST_LOG_OBJECT (ogg, "Pushing %d pending pages after headers",
                   g_slist_length (stream->unknown_pages) + 1);
-
-              for (k = stream->unknown_pages; k != NULL; k = k->next) {
-                GstBuffer *buf;
-
-                buf = GST_BUFFER (k->data);
-                result = gst_pad_push (ogg->srcpad, buf);
+              for (j = 0; j < g_slist_length (stream->unknown_pages); j++) {
+                result = gst_pad_push (ogg->srcpad,
+                    GST_BUFFER (g_slist_nth_data (stream->unknown_pages, j)));
                 if (result != GST_FLOW_OK)
                   return result;
               }
