@@ -54,12 +54,13 @@
 GST_DEBUG_CATEGORY_EXTERN (pango_debug);
 #define GST_CAT_DEFAULT pango_debug
 
-static const GstElementDetails text_render_details =
-GST_ELEMENT_DETAILS ("Text renderer",
-    "Filter/Editor/Video",
-    "Renders a text string to an image bitmap",
-    "David Schleef <ds@schleef.org>, "
-    "Ronald S. Bultje <rbultje@ronald.bitfreak.net>");
+static GstElementDetails text_render_details = {
+  "Text Render",
+  "Filter/Editor/Video",
+  "Renders a text string to an image bitmap",
+  "David Schleef <ds@schleef.org>, "
+      "Ronald S. Bultje <rbultje@ronald.bitfreak.net>"
+};
 
 enum
 {
@@ -286,7 +287,7 @@ gst_text_render_chain (GstPad * pad, GstBuffer * inbuf)
   if (ret != GST_FLOW_OK)
     goto done;
 
-  gst_buffer_copy_metadata (outbuf, inbuf, GST_BUFFER_COPY_TIMESTAMPS);
+  gst_buffer_stamp (outbuf, inbuf);
   data = GST_BUFFER_DATA (outbuf);
 
   for (n = 0; n < render->width * render->height; n++) {
@@ -325,20 +326,18 @@ gst_text_render_finalize (GObject * object)
 static void
 gst_text_render_init (GstTextRender * render, GstTextRenderClass * klass)
 {
-  GstPadTemplate *template;
-
   /* sink */
-  template = gst_static_pad_template_get (&sink_template_factory);
-  render->sinkpad = gst_pad_new_from_template (template, "sink");
-  gst_object_unref (template);
+  render->sinkpad =
+      gst_pad_new_from_template (gst_static_pad_template_get
+      (&sink_template_factory), "sink");
   gst_pad_set_chain_function (render->sinkpad,
       GST_DEBUG_FUNCPTR (gst_text_render_chain));
   gst_element_add_pad (GST_ELEMENT (render), render->sinkpad);
 
   /* source */
-  template = gst_static_pad_template_get (&src_template_factory);
-  render->srcpad = gst_pad_new_from_template (template, "src");
-  gst_object_unref (template);
+  render->srcpad =
+      gst_pad_new_from_template (gst_static_pad_template_get
+      (&src_template_factory), "src");
   gst_pad_set_fixatecaps_function (render->srcpad,
       GST_DEBUG_FUNCPTR (gst_text_render_fixate_caps));
   gst_pad_set_setcaps_function (render->srcpad,
