@@ -21,7 +21,7 @@
 #define __GST_THEORAENC_H__
 
 #include <gst/gst.h>
-#include <theora/theoraenc.h>
+#include <theora/theora.h>
 
 G_BEGIN_DECLS
 
@@ -30,23 +30,15 @@ G_BEGIN_DECLS
 #define GST_THEORA_ENC(obj) \
   (G_TYPE_CHECK_INSTANCE_CAST((obj),GST_TYPE_THEORA_ENC,GstTheoraEnc))
 #define GST_THEORA_ENC_CLASS(klass) \
-  (G_TYPE_CHECK_CLASS_CAST((klass),GST_TYPE_THEORA_ENC,GstTheoraEncClass))
+  (G_TYPE_CHECK_CLASS_CAST((klass),GST_TYPE_THEORA_ENC,GstTheoraEnc))
 #define GST_IS_THEORA_ENC(obj) \
   (G_TYPE_CHECK_INSTANCE_TYPE((obj),GST_TYPE_THEORA_ENC))
-#define GST_IS_THEORA_ENC_CLASS(klass) \
+#define GST_IS_THEORA_ENC_CLASS(obj) \
   (G_TYPE_CHECK_CLASS_TYPE((klass),GST_TYPE_THEORA_ENC))
 
 typedef struct _GstTheoraEnc GstTheoraEnc;
 typedef struct _GstTheoraEncClass GstTheoraEncClass;
 
-/**
- * GstTheoraEncBorderMode:
- * @BORDER_NONE: no border
- * @BORDER_BLACK: black border
- * @BORDER_MIRROR: Mirror image in border
- *
- * Border color to add when sizes not multiple of 16. 
- */ 
 typedef enum
 {
   BORDER_NONE,
@@ -55,11 +47,6 @@ typedef enum
 }
 GstTheoraEncBorderMode;
 
-/**
- * GstTheoraEnc:
- *
- * Opaque data structure.
- */
 struct _GstTheoraEnc
 {
   GstElement element;
@@ -67,42 +54,37 @@ struct _GstTheoraEnc
   GstPad *sinkpad;
   GstPad *srcpad;
 
-  GstSegment segment;
-
   ogg_stream_state to;
 
-  th_enc_ctx *encoder;
-  th_info info;
-  th_comment comment;
-  gboolean initialised;
+  theora_state state;
+  theora_info info;
+  theora_comment comment;
+
+  gboolean center;
+  GstTheoraEncBorderMode border;
 
   gint video_bitrate;           /* bitrate target for Theora video */
   gint video_quality;           /* Theora quality selector 0 = low, 63 = high */
+  gboolean quick;
   gboolean keyframe_auto;
   gint keyframe_freq;
   gint keyframe_force;
+  gint keyframe_threshold;
+  gint keyframe_mindistance;
+  gint noise_sensitivity;
+  gint sharpness;
 
   gint info_width, info_height;
   gint width, height;
+  gint offset_x, offset_y;
   gint fps_n, fps_d;
   GstClockTime next_ts;
 
-  GstClockTime expected_ts;
-  gboolean next_discont;
-
-  gboolean force_keyframe;
-
   guint packetno;
   guint64 bytes_out;
   guint64 granulepos_offset;
   guint64 timestamp_offset;
-
-  gint speed_level;
-  gboolean vp3_compatible;
-  gboolean drop_frames;
-  gboolean cap_overflow;
-  gboolean cap_underflow;
-  int rate_buffer;
+  gint granule_shift;  
 };
 
 struct _GstTheoraEncClass
