@@ -61,48 +61,14 @@ G_BEGIN_DECLS
 #define GST_BASE_AUDIO_SINK_CLASS(klass)        (G_TYPE_CHECK_CLASS_CAST((klass),GST_TYPE_BASE_AUDIO_SINK,GstBaseAudioSinkClass))
 #define GST_BASE_AUDIO_SINK_GET_CLASS(obj)      (G_TYPE_INSTANCE_GET_CLASS ((obj), GST_TYPE_BASE_AUDIO_SINK, GstBaseAudioSinkClass))
 #define GST_IS_BASE_AUDIO_SINK(obj)             (G_TYPE_CHECK_INSTANCE_TYPE((obj),GST_TYPE_BASE_AUDIO_SINK))
-#define GST_IS_BASE_AUDIO_SINK_CLASS(klass)     (G_TYPE_CHECK_CLASS_TYPE((klass),GST_TYPE_BASE_AUDIO_SINK))
+#define GST_IS_BASE_AUDIO_SINK_CLASS(obj)       (G_TYPE_CHECK_CLASS_TYPE((klass),GST_TYPE_BASE_AUDIO_SINK))
 
-/**
- * GST_BASE_AUDIO_SINK_CLOCK:
- * @obj: a #GstBaseAudioSink
- *
- * Get the #GstClock of @obj.
- */
 #define GST_BASE_AUDIO_SINK_CLOCK(obj)   (GST_BASE_AUDIO_SINK (obj)->clock)
-/**
- * GST_BASE_AUDIO_SINK_PAD:
- * @obj: a #GstBaseAudioSink
- *
- * Get the sink #GstPad of @obj.
- */
 #define GST_BASE_AUDIO_SINK_PAD(obj)     (GST_BASE_SINK (obj)->sinkpad)
 
-/**
- * GstBaseAudioSinkSlaveMethod:
- * @GST_BASE_AUDIO_SINK_SLAVE_RESAMPLE: Resample to match the master clock
- * @GST_BASE_AUDIO_SINK_SLAVE_SKEW: Adjust playout pointer when master clock
- * drifts too much.
- * @GST_BASE_AUDIO_SINK_SLAVE_NONE: No adjustment is done. 
- *
- * Different possible clock slaving algorithms
- */
-typedef enum 
-{
-  GST_BASE_AUDIO_SINK_SLAVE_RESAMPLE,
-  GST_BASE_AUDIO_SINK_SLAVE_SKEW,
-  GST_BASE_AUDIO_SINK_SLAVE_NONE
-} GstBaseAudioSinkSlaveMethod;
-
 typedef struct _GstBaseAudioSink GstBaseAudioSink;
 typedef struct _GstBaseAudioSinkClass GstBaseAudioSinkClass;
-typedef struct _GstBaseAudioSinkPrivate GstBaseAudioSinkPrivate;
 
-/**
- * GstBaseAudioSink:
- *
- * Opaque #GstBaseAudioSink.
- */
 struct _GstBaseAudioSink {
   GstBaseSink    element;
 
@@ -110,9 +76,9 @@ struct _GstBaseAudioSink {
   /* our ringbuffer */
   GstRingBuffer *ringbuffer;
 
-  /* required buffer and latency in microseconds */
-  guint64        buffer_time;
-  guint64        latency_time;
+  /* required buffer and latency */
+  GstClockTime   buffer_time;
+  GstClockTime   latency_time;
 
   /* the next sample to write */
   guint64        next_sample;
@@ -122,19 +88,9 @@ struct _GstBaseAudioSink {
   GstClock      *provided_clock;
 
   /*< private >*/
-  GstBaseAudioSinkPrivate *priv;
-
-  gpointer _gst_reserved[GST_PADDING - 1];
+  gpointer _gst_reserved[GST_PADDING];
 };
 
-/**
- * GstBaseAudioSinkClass:
- * @parent_class: the parent class.
- * @create_ringbuffer: create and return a #GstRingBuffer to write to.
- *
- * #GstBaseAudioSink class. Override the vmethod to implement
- * functionality.
- */
 struct _GstBaseAudioSinkClass {
   GstBaseSinkClass parent_class;
 
@@ -149,14 +105,6 @@ GType gst_base_audio_sink_get_type(void);
 
 GstRingBuffer *gst_base_audio_sink_create_ringbuffer (GstBaseAudioSink *sink);
 
-void       gst_base_audio_sink_set_provide_clock      (GstBaseAudioSink *sink, gboolean provide);
-gboolean   gst_base_audio_sink_get_provide_clock      (GstBaseAudioSink *sink);
-
-void       gst_base_audio_sink_set_slave_method       (GstBaseAudioSink *sink, 
-                                                       GstBaseAudioSinkSlaveMethod method);
-GstBaseAudioSinkSlaveMethod
-           gst_base_audio_sink_get_slave_method       (GstBaseAudioSink *sink);
-
 G_END_DECLS
 
 #endif /* __GST_BASE_AUDIO_SINK_H__ */
