@@ -21,32 +21,30 @@
 #ifndef __AUDIORESAMPLE_H__
 #define __AUDIORESAMPLE_H__
 
+
 #include <gst/gst.h>
 #include <gst/base/gstbasetransform.h>
 
 #include "resample.h"
 
+
 G_BEGIN_DECLS
 
+
 #define GST_TYPE_AUDIORESAMPLE \
   (gst_audioresample_get_type())
 #define GST_AUDIORESAMPLE(obj) \
   (G_TYPE_CHECK_INSTANCE_CAST((obj),GST_TYPE_AUDIORESAMPLE,GstAudioresample))
 #define GST_AUDIORESAMPLE_CLASS(klass) \
-  (G_TYPE_CHECK_CLASS_CAST((klass),GST_TYPE_AUDIORESAMPLE,GstAudioresampleClass))
+  (G_TYPE_CHECK_CLASS_CAST((klass),GST_TYPE_AUDIORESAMPLE,GstAudioresample))
 #define GST_IS_AUDIORESAMPLE(obj) \
   (G_TYPE_CHECK_INSTANCE_TYPE((obj),GST_TYPE_AUDIORESAMPLE))
-#define GST_IS_AUDIORESAMPLE_CLASS(klass) \
+#define GST_IS_AUDIORESAMPLE_CLASS(obj) \
   (G_TYPE_CHECK_CLASS_TYPE((klass),GST_TYPE_AUDIORESAMPLE))
 
 typedef struct _GstAudioresample GstAudioresample;
 typedef struct _GstAudioresampleClass GstAudioresampleClass;
 
-/**
- * GstAudioresample:
- *
- * Opaque data structure.
- */
 struct _GstAudioresample {
   GstBaseTransform element;
 
@@ -72,6 +70,8 @@ struct _GstAudioresampleClass {
 
 GType gst_audioresample_get_type(void);
 
+
 G_END_DECLS
 
+
 #endif /* __AUDIORESAMPLE_H__ */
