@@ -1,6 +1,8 @@
 /* GStreamer
  * Copyright (C) 2005 Stefan Kost <ensonic@users.sf.net>
  *
+ * gstaudiotestsrc.c:
+ *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
  * License as published by the Free Software Foundation; either
@@ -31,7 +33,7 @@
  * </para>
  * <para>
  * <programlisting>
- * gst-launch audiotestsrc wave=2 freq=200 ! audioconvert ! tee name=t ! queue ! alsasink t. ! queue ! libvisual_lv_scope ! ffmpegcolorspace ! xvimagesink
+ * gst-launch audiotestsrc wave=2 freq=200 ! audioconvert ! tee name=t ! alsasink t. ! libvisual_lv_scope ! ffmpegcolorspace ! xvimagesink
  * </programlisting>
  * In this example a saw wave is generated. The wave is shown using a
  * scope visualizer from libvisual, allowing you to visually verify that
@@ -51,25 +53,14 @@
 
 #include "gstaudiotestsrc.h"
 
-
-#ifndef M_PI
-#define M_PI  3.14159265358979323846
-#endif
-
-#ifndef M_PI_2
-#define M_PI_2  1.57079632679489661923
-#endif
-
 #define M_PI_M2 ( M_PI + M_PI )
 
-GST_DEBUG_CATEGORY_STATIC (audio_test_src_debug);
-#define GST_CAT_DEFAULT audio_test_src_debug
-
-static const GstElementDetails gst_audio_test_src_details =
-GST_ELEMENT_DETAILS ("Audio test source",
-    "Source/Audio",
-    "Creates audio test signals of given frequency and volume",
-    "Stefan Kost <ensonic@users.sf.net>");
+GstElementDetails gst_audio_test_src_details = {
+  "Audio test source",
+  "Source/Audio",
+  "Creates audio test signals of given frequency and volume",
+  "Stefan Kost <ensonic@users.sf.net>"
+};
 
 
 enum
@@ -104,7 +95,7 @@ static GType
 gst_audiostestsrc_wave_get_type (void)
 {
   static GType audiostestsrc_wave_type = 0;
-  static const GEnumValue audiostestsrc_waves[] = {
+  static GEnumValue audiostestsrc_waves[] = {
     {GST_AUDIO_TEST_SRC_WAVE_SINE, "Sine", "sine"},
     {GST_AUDIO_TEST_SRC_WAVE_SQUARE, "Square", "square"},
     {GST_AUDIO_TEST_SRC_WAVE_SAW, "Saw", "saw"},
@@ -116,7 +107,7 @@ gst_audiostestsrc_wave_get_type (void)
     {0, NULL, NULL},
   };
 
-  if (G_UNLIKELY (audiostestsrc_wave_type == 0)) {
+  if (!audiostestsrc_wave_type) {
     audiostestsrc_wave_type = g_enum_register_static ("GstAudioTestSrcWave",
         audiostestsrc_waves);
   }
@@ -636,7 +627,6 @@ static GstFlowReturn
 gst_audio_test_src_create (GstBaseSrc * basesrc, guint64 offset,
     guint length, GstBuffer ** buffer)
 {
-  GstFlowReturn res;
   GstAudioTestSrc *src;
   GstBuffer *buf;
   GstClockTime next_time;
@@ -647,7 +637,6 @@ gst_audio_test_src_create (GstBaseSrc * basesrc, guint64 offset,
   if (src->eos_reached)
     return GST_FLOW_UNEXPECTED;
 
-  /* example for tagging generated data */
   if (!src->tags_pushed) {
     GstTagList *taglist;
     GstEvent *event;
@@ -662,7 +651,6 @@ gst_audio_test_src_create (GstBaseSrc * basesrc, guint64 offset,
     src->tags_pushed = TRUE;
   }
 
-  /* check for eos */
   if (src->check_seek_stop &&
       (src->n_samples_stop > src->n_samples) &&
       (src->n_samples_stop < src->n_samples + src->samples_per_buffer)
@@ -676,17 +664,15 @@ gst_audio_test_src_create (GstBaseSrc * basesrc, guint64 offset,
     src->generate_samples_per_buffer = src->samples_per_buffer;
     n_samples = src->n_samples + src->samples_per_buffer;
   }
-  next_time = gst_util_uint64_scale (n_samples, GST_SECOND,
-      (guint64) src->samplerate);
+  next_time = n_samples * GST_SECOND / src->samplerate;
 
-  /* allocate a new buffer suitable for this pad */
-  if ((res = gst_pad_alloc_buffer (basesrc->srcpad, src->n_samples,
-              src->generate_samples_per_buffer * sizeof (gint16),
-              GST_PAD_CAPS (basesrc->srcpad), &buf)) != GST_FLOW_OK) {
-    return res;
-  }
+  buf =
+      gst_buffer_new_and_alloc (src->generate_samples_per_buffer *
+      sizeof (gint16));
+  gst_buffer_set_caps (buf, GST_PAD_CAPS (basesrc->srcpad));
 
   GST_BUFFER_TIMESTAMP (buf) = src->timestamp_offset + src->running_time;
+  GST_BUFFER_OFFSET (buf) = src->n_samples;
   GST_BUFFER_OFFSET_END (buf) = n_samples;
   GST_BUFFER_DURATION (buf) = next_time - src->running_time;
 
@@ -772,9 +758,6 @@ plugin_init (GstPlugin * plugin)
   /* initialize gst controller library */
   gst_controller_init (NULL, NULL);
 
-  GST_DEBUG_CATEGORY_INIT (audio_test_src_debug, "audiotestsrc", 0,
-      "Audio Test Source");
-
   return gst_element_register (plugin, "audiotestsrc",
       GST_RANK_NONE, GST_TYPE_AUDIO_TEST_SRC);
 }
