@@ -324,25 +324,71 @@ tta_type_find (GstTypeFind * tf, gpointer unused)
 static GstStaticCaps aac_caps = GST_STATIC_CAPS ("audio/mpeg, "
     "mpegversion = (int) { 2, 4 }, framed = (bool) false");
 #define AAC_CAPS (gst_static_caps_get(&aac_caps))
+#define AAC_AMOUNT (4096)
 static void
 aac_type_find (GstTypeFind * tf, gpointer unused)
 {
-  guint8 *data = gst_type_find_peek (tf, 0, 2);
+  guint8 *data = gst_type_find_peek (tf, 0, AAC_AMOUNT);
+  gint snc;
 
-  /* detect adts header
-   * note that this is a pretty lame typefind method (14 bits, 0.006%), so
-   * we'll only use LIKELY
+  /* detect adts header or adif header.
+   * The ADIF header is 4 bytes, that should be OK. The ADTS header, on
+   * the other hand, is 14 bits only, so we require one valid frame with
+   * again a valid syncpoint on the next one (28 bits) for certainty. We
+   * require 4 kB, which is quite a lot, since frames are generally 200-400
+   * bytes.
    */
   if (data) {
-    if (data[0] == 0xFF && (data[1] & 0xF6) == 0xF0) {
-      gboolean mpegversion = (data[1] & 0x08) ? 2 : 4;
-      GstCaps *caps = gst_caps_new_simple ("audio/mpeg",
-          "framed", G_TYPE_BOOLEAN, FALSE,
-          "mpegversion", G_TYPE_INT, mpegversion,
-          NULL);
+    gint n;
 
-      gst_type_find_suggest (tf, GST_TYPE_FIND_MAXIMUM, caps);
-      gst_caps_unref (caps);
+    for (n = 0; n < AAC_AMOUNT - 3; n++) {
+      snc = GST_READ_UINT16_BE (&data[n]);
+      if ((snc & 0xfff6) == 0xfff0) {
+        /* ADTS header - find frame length */
+        gint len;
+
+        GST_DEBUG ("Found one ADTS syncpoint at offset 0x%x, tracing next...",
+            n);
+        if (AAC_AMOUNT - n < 5) {
+          GST_DEBUG ("Not enough data to parse ADTS header");
+          break;
+        }
+        len = ((data[n + 3] & 0x03) << 11) |
+            (data[n + 4] << 3) | ((data[n + 5] & 0xe0) >> 5);
+        if (n + len + 2 >= AAC_AMOUNT) {
+          GST_DEBUG ("Next frame is not within reach");
+          break;
+        } else if (len == 0) {
+          continue;
+        }
+
+        snc = GST_READ_UINT16_BE (&data[n + len]);
+        if ((snc & 0xfff6) == 0xfff0) {
+          gint mpegversion = (data[n + 1] & 0x08) ? 2 : 4;
+          GstCaps *caps = gst_caps_new_simple ("audio/mpeg",
+              "framed", G_TYPE_BOOLEAN, FALSE,
+              "mpegversion", G_TYPE_INT, mpegversion,
+              NULL);
+
+          gst_type_find_suggest (tf, GST_TYPE_FIND_LIKELY, caps);
+          gst_caps_unref (caps);
+
+          GST_DEBUG ("Found ADTS-%d syncpoint at offset 0x%x (framelen %u)",
+              mpegversion, n, len);
+          break;
+        }
+
+        GST_DEBUG ("No next frame found... (should be at 0x%x)", n + len);
+      } else if (!memcmp (&data[n], "ADIF", 4)) {
+        /* ADIF header */
+        GstCaps *caps = gst_caps_new_simple ("audio/mpeg",
+            "framed", G_TYPE_BOOLEAN, FALSE,
+            "mpegversion", G_TYPE_INT, 4,
+            NULL);
+
+        gst_type_find_suggest (tf, GST_TYPE_FIND_LIKELY, caps);
+        gst_caps_unref (caps);
+      }
     }
   }
 }
@@ -387,7 +433,8 @@ static guint mp3types_freqs[3][3] = { {11025, 12000, 8000},
 
 static inline guint
 mp3_type_frame_length_from_header (guint32 header, guint * put_layer,
-    guint * put_channels, guint * put_bitrate, guint * put_samplerate)
+    guint * put_channels, guint * put_bitrate, guint * put_samplerate,
+    gboolean * may_be_free_format, gint possible_free_framelen)
 {
   guint bitrate, layer, length, mode, samplerate, version, channels;
 
@@ -414,7 +461,11 @@ mp3_type_frame_length_from_header (guint32 header, guint * put_layer,
 
   /* bitrate index */
   bitrate = header & 0xF;
-  if (bitrate == 15 || bitrate == 0)
+  if (bitrate == 0 && possible_free_framelen == -1) {
+    GST_LOG ("Possibly a free format mp3 - signalling");
+    *may_be_free_format = TRUE;
+  }
+  if (bitrate == 15 || (bitrate == 0 && possible_free_framelen == -1))
     return 0;
 
   /* ignore error correction, too */
@@ -433,15 +484,26 @@ mp3_type_frame_length_from_header (guint32 header, guint * put_layer,
 
   /* lookup */
   channels = (mode == 3) ? 1 : 2;
-  bitrate = mp3types_bitrates[version == 3 ? 0 : 1][layer - 1][bitrate];
   samplerate = mp3types_freqs[version > 0 ? version - 1 : 0][samplerate];
-
-  /* calculating */
-  if (layer == 1) {
-    length = ((12000 * bitrate / samplerate) + length) * 4;
+  if (bitrate == 0) {
+    if (layer == 1) {
+      length *= 4;
+      length += possible_free_framelen;
+      bitrate = length * samplerate / 48000;
+    } else {
+      length += possible_free_framelen;
+      bitrate = length * samplerate /
+          ((layer == 3 && version != 3) ? 72000 : 144000);
+    }
   } else {
-    length += ((layer == 3
-            && version != 3) ? 72000 : 144000) * bitrate / samplerate;
+    /* calculating */
+    bitrate = mp3types_bitrates[version == 3 ? 0 : 1][layer - 1][bitrate];
+    if (layer == 1) {
+      length = ((12000 * bitrate / samplerate) + length) * 4;
+    } else {
+      length += ((layer == 3
+              && version != 3) ? 72000 : 144000) * bitrate / samplerate;
+    }
   }
 
   GST_LOG ("mp3typefind: calculated mp3 frame length of %u bytes", length);
@@ -487,10 +549,11 @@ mp3_type_find (GstTypeFind * tf, gpointer unused)
   guint64 skipped;
 
   for (try = 0; try < 2; try++) {
+    gint last_free_offset = -1, last_free_framelen = -1;
     guint64 start_off = (try == 0) ? 0 : length / 2;
 
     if (try != 0 && start_off == 0)
-      return;
+      break;
 
     size = 0;
     skipped = 0;
@@ -515,6 +578,7 @@ mp3_type_find (GstTypeFind * tf, gpointer unused)
           guint length;
           guint prev_layer = 0, prev_bitrate = 0,
               prev_channels = 0, prev_samplerate = 0;
+          gboolean free = FALSE;
 
           if (offset + 4 <= skipped + size) {
             head_data = data + offset - skipped;
@@ -525,11 +589,23 @@ mp3_type_find (GstTypeFind * tf, gpointer unused)
             break;
           head = GST_READ_UINT32_BE (head_data);
           if (!(length = mp3_type_frame_length_from_header (head, &layer,
-                      &channels, &bitrate, &samplerate))) {
+                      &channels, &bitrate, &samplerate, &free,
+                      last_free_framelen))) {
+            if (free) {
+              if (last_free_offset == -1)
+                last_free_offset = offset;
+              else {
+                last_free_framelen = offset - last_free_offset;
+                offset = last_free_offset;
+                continue;
+              }
+            } else {
+              last_free_framelen = -1;
+            }
 
             GST_LOG ("%d. header at offset %" G_GUINT64_FORMAT
-                " (0x%X) was not an mp3 header", found + 1, offset,
-                (guint) offset);
+                " (0x%X) was not an mp3 header (possibly-free: %s)",
+                found + 1, offset, (guint) offset, free ? "yes" : "no");
             break;
           }
           if ((prev_layer && prev_layer != layer) ||
@@ -584,8 +660,9 @@ mp3_type_find (GstTypeFind * tf, gpointer unused)
                 G_TYPE_INT, layer, NULL);
             gst_type_find_suggest (tf, probability, caps);
             gst_caps_unref (caps);
+            return;
           }
-          return;
+          goto no_luck;
         }
       }
       data++;
@@ -593,6 +670,25 @@ mp3_type_find (GstTypeFind * tf, gpointer unused)
       size--;
     }
   }
+
+no_luck:
+
+  /* no luck so far, let's see if there's a valid header right at the start */
+  data = gst_type_find_peek (tf, 0, 4); /* use min. frame size? */
+  if (data) {
+    GstCaps *caps;
+    guint layer;
+
+    if (mp3_type_frame_length_from_header (GST_READ_UINT32_BE (data),
+            &layer, NULL, NULL, NULL, NULL, 0) != 0) {
+      caps = gst_caps_copy (MP3_CAPS);
+      gst_structure_set (gst_caps_get_structure (caps, 0), "layer",
+          G_TYPE_INT, layer, NULL);
+      GST_LOG ("possible mpeg audio layer %u frame at offset 0", layer);
+      gst_type_find_suggest (tf, GST_TYPE_FIND_POSSIBLE - 10, caps);
+      gst_caps_unref (caps);
+    }
+  }
 }
 
 /*** audio/x-ac3 ***/
@@ -638,9 +734,24 @@ wavpack_type_find (GstTypeFind * tf, gpointer unused)
     guint32 blocksize, sublen;
     gint32 left;
 
+    GST_LOG ("got wavpack header");
+
+    /* wavpack blocks can be fairly large, possibly larger than the max.
+     * limits imposed by certain typefinding elements like id3demux or
+     * apedemux. So if the first wavpack block is larger than any particular
+     * limit, we try to get a smaller chunk and hope we get lucky parsing
+     * only bits of it (case at hand: first wavpack block: 42kB, with a
+     * max. limit imposed by apedemux/id3demux: 40kB) */
     blocksize = GST_READ_UINT32_LE (data + 4);
-    /* peek from offset 0, otherwise it won't work with apedemux */
-    data = gst_type_find_peek (tf, 0, 32 + blocksize);
+    do {
+      /* peek from offset 0, otherwise it won't work with apedemux */
+      data = gst_type_find_peek (tf, 0, 32 + blocksize);
+      if (data != NULL)
+        break;
+      if (32 + blocksize < 512) /* random threshold */
+        break;
+      blocksize = (blocksize * 3) / 4;
+    } while (data == NULL);
     if (!data)
       return;
     data += 32;
@@ -981,59 +1092,6 @@ mpeg_video_stream_type_find (GstTypeFind * tf, gpointer unused)
   }
 }
 
-/*** video/quicktime***/
-
-static GstStaticCaps qt_caps = GST_STATIC_CAPS ("video/quicktime");
-
-#define QT_CAPS gst_static_caps_get(&qt_caps)
-#define STRNCMP(x,y,z) (strncmp ((char*)(x), (char*)(y), z))
-
-static void
-qt_type_find (GstTypeFind * tf, gpointer unused)
-{
-  guint8 *data;
-  guint tip = 0;
-  guint64 offset = 0;
-  guint64 size;
-
-  while ((data = gst_type_find_peek (tf, offset, 8)) != NULL) {
-    if (STRNCMP (&data[4], "wide", 4) != 0 &&
-        STRNCMP (&data[4], "moov", 4) != 0 &&
-        STRNCMP (&data[4], "mdat", 4) != 0 &&
-        STRNCMP (&data[4], "pnot", 4) != 0 &&
-        STRNCMP (&data[4], "PICT", 4) != 0 &&
-        STRNCMP (&data[4], "ftyp", 4) != 0 &&
-        STRNCMP (&data[4], "free", 4) != 0 &&
-        STRNCMP (&data[4], "skip", 4) != 0) {
-      tip = 0;
-      break;
-    }
-    if (tip == 0) {
-      tip = GST_TYPE_FIND_LIKELY;
-    } else {
-      tip = GST_TYPE_FIND_MAXIMUM;
-      break;
-    }
-    size = GST_READ_UINT32_BE (data);
-    if (size == 1) {
-      guint8 *sizedata;
-
-      sizedata = gst_type_find_peek (tf, offset + 8, 8);
-      if (sizedata == NULL)
-        break;
-
-      size = GST_READ_UINT64_BE (sizedata);
-    } else {
-      if (size < 8)
-        break;
-    }
-    offset += size;
-  }
-  if (tip > 0) {
-    gst_type_find_suggest (tf, tip, QT_CAPS);
-  }
-};
-
 /*** audio/x-aiff ***/
 
 static GstStaticCaps aiff_caps = GST_STATIC_CAPS ("audio/x-aiff");
@@ -1102,6 +1160,8 @@ ape_type_find (GstTypeFind * tf, gpointer unused)
   }
 }
 
+/*** ISO FORMATS ***/
+
 /*** audio/x-m4a ***/
 
 static GstStaticCaps m4a_caps = GST_STATIC_CAPS ("audio/x-m4a");
@@ -1112,16 +1172,17 @@ m4a_type_find (GstTypeFind * tf, gpointer unused)
 {
   guint8 *data = gst_type_find_peek (tf, 4, 8);
 
-  if (data && memcmp (data, "ftypM4A ", 8) == 0) {
+  if (data &&
+      (memcmp (data, "ftypM4A ", 8) == 0 ||
+          memcmp (data, "ftypmp42", 8) == 0)) {
     gst_type_find_suggest (tf, GST_TYPE_FIND_MAXIMUM, M4A_CAPS);
   }
 }
 
 /*** application/x-3gp ***/
 
-/*
- * The Q is there because variables can't start with a number.
- */
+/* The Q is there because variables can't start with a number. */
+
 
 static GstStaticCaps q3gp_caps = GST_STATIC_CAPS ("application/x-3gp");
 
@@ -1131,11 +1192,68 @@ q3gp_type_find (GstTypeFind * tf, gpointer unused)
 {
   guint8 *data = gst_type_find_peek (tf, 4, 8);
 
-  if (data && memcmp (data, "ftyp3gp4", 8) == 0) {
+  if (data &&
+      (memcmp (data, "ftyp3gp4", 8) == 0 ||
+          memcmp (data, "ftyp3gp5", 8) == 0 ||
+          memcmp (data, "ftyp3gp6", 8) == 0)) {
     gst_type_find_suggest (tf, GST_TYPE_FIND_MAXIMUM, Q3GP_CAPS);
   }
 }
 
+/*** video/quicktime ***/
+
+static GstStaticCaps qt_caps = GST_STATIC_CAPS ("video/quicktime");
+
+#define QT_CAPS gst_static_caps_get(&qt_caps)
+#define STRNCMP(x,y,z) (strncmp ((char*)(x), (char*)(y), z))
+
+static void
+qt_type_find (GstTypeFind * tf, gpointer unused)
+{
+  guint8 *data;
+  guint tip = 0;
+  guint64 offset = 0;
+  guint64 size;
+
+  while ((data = gst_type_find_peek (tf, offset, 8)) != NULL) {
+    if (STRNCMP (&data[4], "wide", 4) != 0 &&
+        STRNCMP (&data[4], "moov", 4) != 0 &&
+        STRNCMP (&data[4], "mdat", 4) != 0 &&
+        STRNCMP (&data[4], "pnot", 4) != 0 &&
+        STRNCMP (&data[4], "PICT", 4) != 0 &&
+        STRNCMP (&data[4], "ftyp", 4) != 0 &&
+        STRNCMP (&data[4], "free", 4) != 0 &&
+        STRNCMP (&data[4], "skip", 4) != 0) {
+      tip = 0;
+      break;
+    }
+    if (tip == 0) {
+      tip = GST_TYPE_FIND_LIKELY;
+    } else {
+      tip = GST_TYPE_FIND_MAXIMUM;
+      break;
+    }
+    size = GST_READ_UINT32_BE (data);
+    if (size == 1) {
+      guint8 *sizedata;
+
+      sizedata = gst_type_find_peek (tf, offset + 8, 8);
+      if (sizedata == NULL)
+        break;
+
+      size = GST_READ_UINT64_BE (sizedata);
+    } else {
+      if (size < 8)
+        break;
+    }
+    offset += size;
+  }
+  if (tip > 0) {
+    gst_type_find_suggest (tf, tip, QT_CAPS);
+  }
+};
+
+
 /*** audio/x-mod ***/
 
 static GstStaticCaps mod_caps = GST_STATIC_CAPS ("audio/x-mod");
@@ -1384,7 +1502,7 @@ ircam_type_find (GstTypeFind * tf, gpointer ununsed)
     gst_type_find_suggest (tf, GST_TYPE_FIND_MAXIMUM, IRCAM_CAPS);
     return;
   }
-  // now try the reverse version
+  /* now try the reverse version */
   matched = TRUE;
   for (x = 0; x < 4; x++) {
     if ((data[x] & mask[3 - x]) != match[3 - x]) {
@@ -1749,6 +1867,61 @@ ar_type_find (GstTypeFind * tf, gpointer unused)
   }
 }
 
+/*** audio/x-au ***/
+
+/* NOTE: we cannot replace this function with TYPE_FIND_REGISTER_START_WITH,
+ * as it is only possible to register one typefind factory per 'name'
+ * (which is in this case the caps), and the first one would be replaced by
+ * the second one. */
+static GstStaticCaps au_caps = GST_STATIC_CAPS ("audio/x-au");
+
+#define AU_CAPS (gst_static_caps_get(&au_caps))
+static void
+au_type_find (GstTypeFind * tf, gpointer unused)
+{
+  guint8 *data = gst_type_find_peek (tf, 0, 4);
+
+  if (data) {
+    if (memcmp (data, ".snd", 4) == 0 || memcmp (data, "dns.", 4) == 0) {
+      gst_type_find_suggest (tf, GST_TYPE_FIND_MAXIMUM, AU_CAPS);
+    }
+  }
+}
+
+/*** audio/x-paris ***/
+/* NOTE: do not replace this function with two TYPE_FIND_REGISTER_START_WITH */
+static GstStaticCaps paris_caps = GST_STATIC_CAPS ("audio/x-paris");
+
+#define PARIS_CAPS (gst_static_caps_get(&paris_caps))
+static void
+paris_type_find (GstTypeFind * tf, gpointer unused)
+{
+  guint8 *data = gst_type_find_peek (tf, 0, 4);
+
+  if (data) {
+    if (memcmp (data, " paf", 4) == 0 || memcmp (data, "fap ", 4) == 0) {
+      gst_type_find_suggest (tf, GST_TYPE_FIND_MAXIMUM, PARIS_CAPS);
+    }
+  }
+}
+
+/*** audio/iLBC-sh ***/
+/* NOTE: do not replace this function with two TYPE_FIND_REGISTER_START_WITH */
+static GstStaticCaps ilbc_caps = GST_STATIC_CAPS ("audio/iLBC-sh");
+
+#define ILBC_CAPS (gst_static_caps_get(&ilbc_caps))
+static void
+ilbc_type_find (GstTypeFind * tf, gpointer unused)
+{
+  guint8 *data = gst_type_find_peek (tf, 0, 8);
+
+  if (data) {
+    if (memcmp (data, "#!iLBC30", 8) == 0 || memcmp (data, "#!iLBC20", 8) == 0) {
+      gst_type_find_suggest (tf, GST_TYPE_FIND_LIKELY, ILBC_CAPS);
+    }
+  }
+}
+
 /*** application/x-ms-dos-executable ***/
 
 static GstStaticCaps msdos_caps =
@@ -1860,7 +2033,7 @@ plugin_init (GstPlugin * plugin)
   static gchar *flx_exts[] = { "flc", "fli", NULL };
   static gchar *id3_exts[] =
       { "mp3", "mp2", "mp1", "mpga", "ogg", "flac", "tta", NULL };
-  static gchar *apetag_exts[] = { "ape", "mpc", NULL };
+  static gchar *apetag_exts[] = { "ape", "mpc", "wv", NULL };   /* and mp3 and wav? */
   static gchar *tta_exts[] = { "tta", NULL };
   static gchar *mod_exts[] = { "669", "amf", "dsm", "gdm", "far", "imf",
     "it", "med", "mod", "mtm", "okt", "sam",
@@ -1933,10 +2106,8 @@ plugin_init (GstPlugin * plugin)
   /* -1 so id3v1 or apev1/2 are detected with higher preference */
   TYPE_FIND_REGISTER_START_WITH (plugin, "audio/x-musepack", GST_RANK_PRIMARY,
       musepack_exts, "MP+", 3, GST_TYPE_FIND_MAXIMUM - 1);
-  TYPE_FIND_REGISTER_START_WITH (plugin, "audio/x-au", GST_RANK_MARGINAL,
-      au_exts, ".snd", 4, GST_TYPE_FIND_MAXIMUM);
-  TYPE_FIND_REGISTER_START_WITH (plugin, "audio/x-au", GST_RANK_MARGINAL,
-      au_exts, "dns.", 4, GST_TYPE_FIND_MAXIMUM);
+  TYPE_FIND_REGISTER (plugin, "audio/x-au", GST_RANK_MARGINAL,
+      au_type_find, au_exts, AU_CAPS, NULL, NULL);
   TYPE_FIND_REGISTER_RIFF (plugin, "video/x-msvideo", GST_RANK_PRIMARY,
       avi_exts, "AVI ");
   TYPE_FIND_REGISTER_RIFF (plugin, "video/x-cdxa", GST_RANK_PRIMARY,
@@ -1971,8 +2142,15 @@ plugin_init (GstPlugin * plugin)
   TYPE_FIND_REGISTER (plugin, "video/mpeg-stream", GST_RANK_MARGINAL,
       mpeg_video_stream_type_find, mpeg_video_exts, MPEG_VIDEO_CAPS, NULL,
       NULL);
+
+  /* ISO formats */
+  TYPE_FIND_REGISTER (plugin, "audio/x-m4a", GST_RANK_PRIMARY, m4a_type_find,
+      m4a_exts, M4A_CAPS, NULL, NULL);
+  TYPE_FIND_REGISTER (plugin, "application/x-3gp", GST_RANK_PRIMARY,
+      q3gp_type_find, q3gp_exts, Q3GP_CAPS, NULL, NULL);
   TYPE_FIND_REGISTER (plugin, "video/quicktime", GST_RANK_SECONDARY,
       qt_type_find, qt_exts, QT_CAPS, NULL, NULL);
+
   TYPE_FIND_REGISTER_START_WITH (plugin, "application/vnd.rn-realmedia",
       GST_RANK_SECONDARY, rm_exts, ".RMF", 4, GST_TYPE_FIND_MAXIMUM);
   TYPE_FIND_REGISTER (plugin, "application/x-shockwave-flash",
@@ -1991,10 +2169,8 @@ plugin_init (GstPlugin * plugin)
       aiff_type_find, aiff_exts, AIFF_CAPS, NULL, NULL);
   TYPE_FIND_REGISTER (plugin, "audio/x-svx", GST_RANK_SECONDARY, svx_type_find,
       svx_exts, SVX_CAPS, NULL, NULL);
-  TYPE_FIND_REGISTER_START_WITH (plugin, "audio/x-paris", GST_RANK_SECONDARY,
-      paris_exts, " paf", 4, GST_TYPE_FIND_MAXIMUM);
-  TYPE_FIND_REGISTER_START_WITH (plugin, "audio/x-paris", GST_RANK_SECONDARY,
-      paris_exts, "fap ", 4, GST_TYPE_FIND_MAXIMUM);
+  TYPE_FIND_REGISTER (plugin, "audio/x-paris", GST_RANK_SECONDARY,
+      paris_type_find, paris_exts, PARIS_CAPS, NULL, NULL);
   TYPE_FIND_REGISTER_START_WITH (plugin, "audio/x-nist", GST_RANK_SECONDARY,
       nist_exts, "NIST", 4, GST_TYPE_FIND_MAXIMUM);
   TYPE_FIND_REGISTER_START_WITH (plugin, "audio/x-voc", GST_RANK_SECONDARY,
@@ -2027,10 +2203,8 @@ plugin_init (GstPlugin * plugin)
       amr_exts, "#!AMR", 5, GST_TYPE_FIND_LIKELY);
   TYPE_FIND_REGISTER_START_WITH (plugin, "audio/x-amr-wb-sh", GST_RANK_PRIMARY,
       amr_exts, "#!AMR-WB", 7, GST_TYPE_FIND_MAXIMUM);
-  TYPE_FIND_REGISTER_START_WITH (plugin, "audio/iLBC-sh", GST_RANK_PRIMARY,
-      ilbc_exts, "#!iLBC30", 8, GST_TYPE_FIND_LIKELY);
-  TYPE_FIND_REGISTER_START_WITH (plugin, "audio/iLBC-sh", GST_RANK_PRIMARY,
-      ilbc_exts, "#!iLBC20", 8, GST_TYPE_FIND_LIKELY);
+  TYPE_FIND_REGISTER (plugin, "audio/iLBC-sh", GST_RANK_PRIMARY,
+      ilbc_type_find, ilbc_exts, ILBC_CAPS, NULL, NULL);
   TYPE_FIND_REGISTER_START_WITH (plugin, "audio/x-sid", GST_RANK_MARGINAL,
       sid_exts, "PSID", 4, GST_TYPE_FIND_MAXIMUM);
   TYPE_FIND_REGISTER_START_WITH (plugin, "image/x-xcf", GST_RANK_SECONDARY,
@@ -2068,10 +2242,6 @@ plugin_init (GstPlugin * plugin)
       oggskel_type_find, NULL, OGG_SKELETON_CAPS, NULL, NULL);
   TYPE_FIND_REGISTER (plugin, "text/x-cmml", GST_RANK_PRIMARY, cmml_type_find,
       NULL, CMML_CAPS, NULL, NULL);
-  TYPE_FIND_REGISTER (plugin, "audio/x-m4a", GST_RANK_PRIMARY, m4a_type_find,
-      m4a_exts, M4A_CAPS, NULL, NULL);
-  TYPE_FIND_REGISTER (plugin, "application/x-3gp", GST_RANK_PRIMARY,
-      q3gp_type_find, q3gp_exts, Q3GP_CAPS, NULL, NULL);
   TYPE_FIND_REGISTER_START_WITH (plugin, "application/x-executable",
       GST_RANK_MARGINAL, NULL, "\177ELF", 4, GST_TYPE_FIND_MAXIMUM);
   TYPE_FIND_REGISTER (plugin, "adts_mpeg_stream", GST_RANK_SECONDARY,
@@ -2091,6 +2261,8 @@ plugin_init (GstPlugin * plugin)
       ar_type_find, ar_exts, AR_CAPS, NULL, NULL);
   TYPE_FIND_REGISTER (plugin, "application/x-ms-dos-executable",
       GST_RANK_SECONDARY, msdos_type_find, msdos_exts, MSDOS_CAPS, NULL, NULL);
+  TYPE_FIND_REGISTER_START_WITH (plugin, "video/x-dirac",
+      GST_RANK_PRIMARY, NULL, "BBCD", 4, GST_TYPE_FIND_LIKELY);
 
   return TRUE;
 }
