@@ -24,15 +24,9 @@
  * <refsect2>
  * <title>Example launch line</title>
  * <para>
- * The videotestsrc element is used to produce test video data in a wide variaty
- * of formats. The video test data produced can be controlled with the "pattern"
- * property.
- * </para>
- * <para>
  * <programlisting>
  * gst-launch -v videotestsrc pattern=snow ! ximagesink
  * </programlisting>
- * Shows random noise in an X window.
  * </para>
  * </refsect2>
  */
@@ -49,11 +43,11 @@
 
 #define USE_PEER_BUFFERALLOC
 
-GST_DEBUG_CATEGORY_STATIC (video_test_src_debug);
+GST_DEBUG_CATEGORY (video_test_src_debug);
 #define GST_CAT_DEFAULT video_test_src_debug
 
 
-static const GstElementDetails video_test_src_details =
+static GstElementDetails video_test_src_details =
 GST_ELEMENT_DETAILS ("Video test source",
     "Source/Video",
     "Creates a test video stream",
@@ -101,15 +95,11 @@ static GType
 gst_video_test_src_pattern_get_type (void)
 {
   static GType video_test_src_pattern_type = 0;
-  static const GEnumValue pattern_types[] = {
+  static GEnumValue pattern_types[] = {
     {GST_VIDEO_TEST_SRC_SMPTE, "SMPTE 100% color bars", "smpte"},
     {GST_VIDEO_TEST_SRC_SNOW, "Random (television snow)", "snow"},
     {GST_VIDEO_TEST_SRC_BLACK, "100% Black", "black"},
-    {GST_VIDEO_TEST_SRC_WHITE, "100% White", "white"},
-    {GST_VIDEO_TEST_SRC_RED, "Red", "red"},
-    {GST_VIDEO_TEST_SRC_GREEN, "Green", "green"},
-    {GST_VIDEO_TEST_SRC_BLUE, "Blue", "blue"},
-    {0, NULL, NULL}
+    {0, NULL, NULL},
   };
 
   if (!video_test_src_pattern_type) {
@@ -145,11 +135,11 @@ gst_video_test_src_class_init (GstVideoTestSrcClass * klass)
   gobject_class->set_property = gst_video_test_src_set_property;
   gobject_class->get_property = gst_video_test_src_get_property;
 
-  g_object_class_install_property (gobject_class, PROP_PATTERN,
+  g_object_class_install_property (G_OBJECT_CLASS (klass), PROP_PATTERN,
       g_param_spec_enum ("pattern", "Pattern",
           "Type of test pattern to generate", GST_TYPE_VIDEO_TEST_SRC_PATTERN,
           1, G_PARAM_READWRITE));
-  g_object_class_install_property (gobject_class,
+  g_object_class_install_property (G_OBJECT_CLASS (klass),
       PROP_TIMESTAMP_OFFSET, g_param_spec_int64 ("timestamp-offset",
           "Timestamp offset",
           "An offset added to timestamps set on buffers (in ns)", G_MININT64,
@@ -215,18 +205,6 @@ gst_video_test_src_set_pattern (GstVideoTestSrc * videotestsrc,
     case GST_VIDEO_TEST_SRC_BLACK:
       videotestsrc->make_image = gst_video_test_src_black;
       break;
-    case GST_VIDEO_TEST_SRC_WHITE:
-      videotestsrc->make_image = gst_video_test_src_white;
-      break;
-    case GST_VIDEO_TEST_SRC_RED:
-      videotestsrc->make_image = gst_video_test_src_red;
-      break;
-    case GST_VIDEO_TEST_SRC_GREEN:
-      videotestsrc->make_image = gst_video_test_src_green;
-      break;
-    case GST_VIDEO_TEST_SRC_BLUE:
-      videotestsrc->make_image = gst_video_test_src_blue;
-      break;
     default:
       g_assert_not_reached ();
   }
@@ -399,12 +377,8 @@ gst_video_test_src_query (GstBaseSrc * bsrc, GstQuery * query)
           switch (dest_fmt) {
             case GST_FORMAT_TIME:
               /* frames to time */
-              if (src->rate_numerator) {
-                dest_val = gst_util_uint64_scale (src_val,
-                    src->rate_denominator * GST_SECOND, src->rate_numerator);
-              } else {
-                dest_val = 0;
-              }
+              dest_val = gst_util_uint64_scale (src_val,
+                  src->rate_denominator * GST_SECOND, src->rate_numerator);
               break;
             default:
               goto error;
@@ -414,12 +388,8 @@ gst_video_test_src_query (GstBaseSrc * bsrc, GstQuery * query)
           switch (dest_fmt) {
             case GST_FORMAT_DEFAULT:
               /* time to frames */
-              if (src->rate_numerator) {
-                dest_val = gst_util_uint64_scale (src_val,
-                    src->rate_numerator, src->rate_denominator * GST_SECOND);
-              } else {
-                dest_val = 0;
-              }
+              dest_val = gst_util_uint64_scale (src_val,
+                  src->rate_numerator, src->rate_denominator * GST_SECOND);
               break;
             default:
               goto error;
@@ -480,19 +450,10 @@ gst_video_test_src_do_seek (GstBaseSrc * bsrc, GstSegment * segment)
   time = segment->time = segment->start;
 
   /* now move to the time indicated */
-  if (src->rate_numerator) {
-    src->n_frames = gst_util_uint64_scale (time,
-        src->rate_numerator, src->rate_denominator * GST_SECOND);
-  } else {
-    src->n_frames = 0;
-  }
-  if (src->rate_numerator) {
-    src->running_time = gst_util_uint64_scale (src->n_frames,
-        src->rate_denominator * GST_SECOND, src->rate_numerator);
-  } else {
-    /* FIXME : Not sure what to set here */
-    src->running_time = 0;
-  }
+  src->n_frames = gst_util_uint64_scale (time,
+      src->rate_numerator, src->rate_denominator * GST_SECOND);
+  src->running_time = gst_util_uint64_scale (src->n_frames,
+      src->rate_denominator * GST_SECOND, src->rate_numerator);
 
   g_assert (src->running_time <= time);
 
@@ -517,13 +478,9 @@ gst_video_test_src_create (GstPushSrc * psrc, GstBuffer ** buffer)
 
   src = GST_VIDEO_TEST_SRC (psrc);
 
-  if (src->fourcc == NULL)
+  if (src->fourcc == NULL || src->rate_numerator == 0)
     goto not_negotiated;
 
-  /* 0 framerate and we are at the second frame, eos */
-  if (src->rate_numerator == 0 && src->n_frames == 1)
-    goto eos;
-
   newsize = gst_video_test_src_get_size (src, src->width, src->height);
 
   g_return_val_if_fail (newsize > 0, GST_FLOW_ERROR);
@@ -550,15 +507,9 @@ gst_video_test_src_create (GstPushSrc * psrc, GstBuffer ** buffer)
   GST_BUFFER_OFFSET (outbuf) = src->n_frames;
   src->n_frames++;
   GST_BUFFER_OFFSET_END (outbuf) = src->n_frames;
-  if (src->rate_numerator) {
-    next_time = gst_util_uint64_scale_int (src->n_frames * GST_SECOND,
-        src->rate_denominator, src->rate_numerator);
-    GST_BUFFER_DURATION (outbuf) = next_time - src->running_time;
-  } else {
-    next_time = src->timestamp_offset;
-    /* NONE means forever */
-    GST_BUFFER_DURATION (outbuf) = GST_CLOCK_TIME_NONE;
-  }
+  next_time = gst_util_uint64_scale_int (src->n_frames * GST_SECOND,
+      src->rate_denominator, src->rate_numerator);
+  GST_BUFFER_DURATION (outbuf) = next_time - src->running_time;
 
   src->running_time = next_time;
 
@@ -572,11 +523,6 @@ not_negotiated:
         ("format wasn't negotiated before get function"));
     return GST_FLOW_NOT_NEGOTIATED;
   }
-eos:
-  {
-    GST_DEBUG_OBJECT (src, "eos: 0 framerate and frame %d", src->n_frames);
-    return GST_FLOW_UNEXPECTED;
-  }
 no_buffer:
   {
     GST_DEBUG_OBJECT (src, "could not allocate buffer, reason %s",
