@@ -485,6 +485,11 @@ gst_ximagesink_ximage_destroy (GstXImageSink * ximagesink,
     ximagesink->cur_image = NULL;
   }
 
+  /* We might have some buffers destroyed after changing state to NULL */
+  if (!ximagesink->xcontext) {
+    goto beach;
+  }
+
   g_mutex_lock (ximagesink->x_lock);
 
 #ifdef HAVE_XSHM
@@ -511,6 +516,7 @@ gst_ximagesink_ximage_destroy (GstXImageSink * ximagesink,
 
   g_mutex_unlock (ximagesink->x_lock);
 
+beach:
   if (ximage->ximagesink) {
     /* Release the ref to our sink */
     ximage->ximagesink = NULL;
@@ -919,7 +925,10 @@ gst_ximagesink_calculate_pixel_aspect_ratio (GstXContext * xcontext)
     {1, 1},                     /* regular screen */
     {16, 15},                   /* PAL TV */
     {11, 10},                   /* 525 line Rec.601 video */
-    {54, 59}                    /* 625 line Rec.601 video */
+    {54, 59},                   /* 625 line Rec.601 video */
+    {64, 45},                   /* 1280x1024 on 16:9 display */
+    {5, 3},                     /* 1280x1024 on 4:3 display */
+    {4, 3}                      /*  800x600 on 16:9 display */
   };
   gint i;
   gint index;
